\name{AdviaData}
\alias{AdviaData}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Data from Advia Mouse Cross}
\description{
  A Sample File for analyzing Advia Mouse data for use with DiallelAnalyze
    or FullDiallelAnalyze object
}
\usage{
 data(AdviaData)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mother.strain.name}{Character Strains for Mothers}
  \item{father.strain.name}{Character Strains for Father of Test Subject}
  \item{is.female}{1 for if female, 0 if male}
  \item{Mouse.ID}{ID number for test subject}
  \item{Name}{Mouse name by cross}
  \item{cHGB}{HGB concentration}
  \item{mHGB}{HGB measure}
  \item{EOS}{Percent Eosinophils}
  \item{WBC}{White Blood Cell Count}
}
\details{
 These health phenotypes data demonstrate basic capabilities of BayesDiallel

}
\value{
    Object  
     \item{FullDiallelAnalyze}{Returns R.oo class that fits model}
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{ \code{\link{FullDiallelAnalyze}}, 
  \code{\link{DiallelAnalyzer}}, \code{\link{DiallelOb}},
  \code{\link{PiximusData}}, \code{\link{ChemData}} }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
try(require(BayesDiallelGraphics, warn.conflicts=FALSE, quietly=TRUE));

## Trial File

SaveAFDFile = ""
###############################################################################
##  To Save AFD, consider setting SaveAFDFile to accessible directory location
##  SaveAFDFile = "c:/Stat/AFDSaves/AFDSaveAdvia.RData"
##  SaveAFDFile = "~/Documents/AFDSaves/AFDSaveAdvia.RData"

data(AdviaData);
##  Data is in "Advia.Data"
##  Models are in "Examples.Advia.Models"
##  tau.Prior is in "Examples.Advia.tau.Prior.Info"
##DataFile <- paste(AHome, "/AdviaData.txt", sep="");
##ModelFile <- paste(AHome, "/ModelsFile.txt", sep="");
##TauData <- rbind(c(.2,.2,.2,.2,.2,.2),c(.2,.2,.2,.2,.2,.2));
##colnames(TauData) <- c("aj", "dominancej", "genderj", "symcrossjk",
##  "asymcrossjkDkj", "motherj");
##rownames(TauData) <- c("m", "df")
##tauPriorFile <- read.table(paste(AHome, "/tauPriorFile.txt", sep=""), sep="");
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Advia.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="WBC", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Advia.Models[1:2], sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Advia.tau.Prior.Info,
   SaveAFDFile = SaveAFDFile);

###########################################################################
## Note: If loading Saved entry from previous R-instance,
## load-in library(R.oo);library(BayesDiallel) first
## library(R.oo); library(BayesDiallel);  load(SaveAFDFile);
##
##   SaveAFDFile will always save the DiallelAnalyzer run to object named AFD;


## AFD is the FullDiallelAnalyze object (contains all models)
##  AO is the DiallelObs for the first model;
AO = AFD$AllDiallelObs[[1]];  ##  AO is the first member of all AllDiallelObs of AFD;

## Valid Alternative method to get same AO
AFD$OnObs = 1;  AO = AFD$On;
##  When AFD$On is set to the right iteration of desired AO, 
##    many DiallelOb options function directly on AFD;


## Chains are the first version, they are preshrunk towards group means.  You should
##   often use these by default;
summary(AO$cent.chains);   ## summary of the chains;
plot(AO$cent.chains);      ## Use Coda MCMC to plot the many member of chains consider "ask"/"wanted"
plot.hpd(AO$cent.chains);  ## Plot Hpd intervals for members of "chains" object.  Consider "ask"/"wanted"
plot.hpd(AO$cent.chains, wanted = 1:20);  ## Plot HPD intervals for first 20 members


  
## CodaChains are the "unshrunk version"
## Get a summary of the Coda Chains for first Diallel Model
summary(AFD$AllDiallelObs[[1]]$raw.chains)
plot(AFD$AllDiallelObs[[1]]$raw.chains);
plot.hpd(AFD$AllDiallelObs[[1]]$raw.chains);

ADO = AFD$AllDiallelObs[[1]];

plot(ADO$raw.chains);  ## Plot the chains to understand
summary(ADO$raw.chains);  ## Get Summary information on a parameters
    
## To use plot.hpd, one needs package BayesDiallelGraphics                                      
plot.hpd(ADO$raw.chains); ## Using Library_diallel_helper, plot chains.

plot.corr(crosscorr(ADO$raw.chains)) ## Look at cross correlations of coda chains
plot.corr(crosscorr(ADO$cent.chains))  ## representative correlations of Mean shrunken Chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain Model 

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

##SimmedGrid = GiveGrid(AFD);
## SimmedGrid only works if you have real values!


## Only works if "Female/Male" model difference exists in ADO
par(mfrow=c(1,2));
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior", main="Females")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 0,], phenotype = "Mean Posterior", main="Males")

PD = as.data.frame(cbind(Advia.Data$"WBC",Advia.Data$"mother.strain.name",Advia.Data$"father.strain.name",
  Advia.Data$"is.female"), stringsAsFactors=FALSE); 
colnames(PD) = c("White Blood Cells (WBC)", "mother.strain", "father.strain", "is.female");
TwoDrawObject(DT1=PD[PD$is.female,], 
  DT2=MyPostSum[MyPostSum$"Female/Male" == 1,], data.range=NULL,
  phenotype1 = "White Blood Cells (WBC)", phenotype2 = "Mean Posterior", strain.names=AFD$strain.map[,1],
  main1 = "(Females-observed)", main2="(Females-predicted)", cols = NULL, 
  col.smoothness = 20, npt = 6, roundDP = 2, HeadTitle = "Advia Data")
  
TwoDrawObject(DT1=MyPostSum[MyPostSum$"Female/Male" == 1,], DT2=MyPostSum[MyPostSum$"Female/Male" == 0,], data.range=NULL,
  phenotype="Mean Posterior",
  phenotype1 = NULL, phenotype2 = NULL, strain.names=AFD$strain.map[,],
  main1 = "(Females)", main2="(Males)", cols = NULL, 
  col.smoothness = 20, npt = 6, roundDP = 2, HeadTitle = "Advia Data")
  
DT1=MyPostSum[MyPostSum$"Female/Male" == 1,]; DT2=MyPostSum[MyPostSum$"Female/Male" == 0,]; data.range=NULL;
  phenotype="Mean Posterior";
  phenotype1 = NULL; phenotype2 = NULL; strain.names=AFD$strain.map[,1];
  main1 = "(Females)"; main2="(Males)"; cols = NULL; 
  col.smoothness = 2; npt = 6; roundDP = 2; HeadTitle = "Advia Data"; 

## Works if there is no sex strain  
PD = as.data.frame(cbind(Advia.Data$"WBC",as.character(Advia.Data$"mother.strain.name"),
  as.character(Advia.Data$"father.strain.name")),stringsAsFactors =FALSE); 
colnames(PD) = c("White Blood Cell (WBC)", "mother.strain", "father.strain");
TwoDrawObject(DT1=PD, 
  DT2=MyPostSum, data.range=NULL,
  phenotype1 = "White Blood Cell (WBC)", phenotype2 = "Mean Posterior", strain.names=AFD$strain.map[,1],
  main1 = "(observed)", main2="(predicted)", cols = NULL, 
  col.smoothness = 20, npt = 6, roundDP = 2, HeadTitle = "Advia Data")
DT1=PD; DT2=MyPostSum; data.range=NULL;
  phenotype=NULL;
  phenotype1 = "White Blood Cell (WBC)"; phenotype2 = "Mean Posterior"; strain.names=AFD$strain.map[,1];
  main1 = "(observed)"; main2="(predicted)"; cols = NULL; 
  col.smoothness = 200; npt = 6; roundDP = 2; HeadTitle = "Advia Data";  

}


% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
