\name{BayesDiallelHiddenDetails}
\alias{BayesDiallelHiddenDetails}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{BayesDiallel Package, additional hidden details}
\description{

  This is some extended details using flags system used in interior of \code{\link{BayesDiallel}}'
   given package R.oo ("\code{\link[R.oo]{setConstructorS3}}")'s objects.  \code{\link{ModelsFile}} gives description of desired
   ModelsFile formats.  \code{\link{ReadModels}} is function which converts a models file to desired models text.
   
   
  Package Designed to explore Diallel Data using hierarchical Bayes mixed-model
    gibbs sampling.
    
  Function "\code{\link{DiallelAnalyzer}}"  and Object "\code{\link{FullDiallelAnalyze}}" are key pieces of
   info on details of operating mechanisms.  
   
   "\code{\link{DiallelAnalyzer}}" is key wrapper function for which inputs
   \code{data} (with a list of father stains, mother strains, sex of subjects, and a real valued phenotype [Options for 
   Fixed Effects (drug or treatment) and Random Effects Exists (batch effects...)])) and \code{Models} (a list of possible
   diallel interaction models to explore are submitted, default is "Fulls" full sexed model.)
   
   A "\code{\link{FullDiallelAnalyze}}" object is the output.  Many summary and posterior analysis options exist for this object,
   including options to generate in posterior precition plots.
   
  By "Diallel" - we mean the phenotype analysis of an F1 cross of a number of parent strains.  This analysis technique separates the observed 
  phenotypes into  types of effects "additive", "maternal parent of origin", "dominance inbreeding penalty", 
  and symmetric and anti-symmetric cross specific epistasis interactions.  In general, this is a linear model:
  
  Y_i(mother strain=j, father strain=k) ~ additive_effects + maternal_effects + inbreeding_effects + epistatsis_effects + i.i.d. error
  
  When phenotype shows sex-specific effects, sex interaction terms for all effects are included.  Default i.i.d. error is N(0,sigma^2),
  but t-distributed long tailed error cand be inspected by setting flag \code{dfTNoise}.  

}
\usage{
  DiallelAnalyzer(data = NULL,
  father.strain="father.strain",
  mother.strain="mother.strain",
  phenotype="phenotype",
  is.female="is_female",
  FixedEffects=NULL,
  RandomEffects=NULL,
  Models=NULL,
  sep=",", na.strings="NA", 
  sigmasq.start = 1, numChains =3, lengthChains = 2000,
  Verbose = TRUE,
  burnin = 1, thin = 1, DIC.Only = FALSE,
  TauFile = NULL);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{Data frame containing data, every row is a diallel subject, else a Filename where file is separated by "sep"}
  \item{phenotype}{Name of column in "DataFile" that is the dependent phenotype}
  \item{father.strain}{Name of column in "DataFile" that is the father strain indicator, could be numeric or character, reduced to a factor}
  \item{mother.strain}{Name of column in "DataFile" that is the mother strain indicator, could be numeric or character, reduced to a factor}
  \item{is.female}{Name of column in "DataFile" that records gender of the subject}    
  \item{FixedEffects}{Column or Columns of Data (in DataFile) with additional fixed effects in recorded}
  \item{RandomEffects}{Column or Columns of Data (in DataFile) with additional Random Effects (factors) if recorded}
  \item{nameFixedEffects}{Name to give columns of Fixed Effects to be stored for column name}
  \item{nameRandomEffects}{Name to be given to Random Effects Groups}
  \item{Models}{Character vector, or matrix of model values describing models or superset of models of interest}
  \item{sep}{ Separator of columns if DataFile is a file to be read in}
  \item{na.strings}{Character set representing "NA" value for unknowns in the table}
  \item{Sigma.start}{Staring Value of Noise to be used in chains (will be learned in Gibbs Sampling}
  \item{lengthChains}{ Length of Gibbs Sampler Chains}
  \item{numChains}{Number of Gibbs Sampler Chains}
  \item{Verbose}{Give feedback during the operation?}
  \item{burnin.start}{Automatically kill chains starting up untill burnin.start}
  \item{burnin.end}{Kill chains at end of burnin.end, default length of chains}
  \item{thin}{Thin chains by deleting rows every thin amount}
  \item{DIC.only}{Report only the DIC of the models tested, delete chains as run for more memory}
  \item{TauFile}{An Optional file of Mean and degree of freedom for inverse-gamma priors 
    for the various groups in the algorithm (including additional Random Effects}
}
\details{
  \code{\link{FullDiallel}} objects come in two types, \code{\link{FullDiallelSimulate}},
    and \code{\link{FullDiallelAnalyze}}.  Analyses will be done with \code{FullDiallelAnalyze} objects.

  \code{\link{DiallelOb}} has several default functions including "Copy" and "finalize" for 
  deleting.  
  
  
  
\cr Hidden Details: The many diallel models are toggled by the setting of 6 independent flags. At the setting of the flags,
 the \code{BayesDiallel} method generates an "X" covariate matrix out of the "Listjk" parental-strain-names vector that
 features the effects specific to a Diallel Model.
  
  Hidden members .numj (number of diallel strains), .ajModel, .MotherModel,
    .CrossModel develop from model cross levels
    .SexModel, .BetaInbredLevel, .BetaInbredLevelTimesSex
    
  DiallelObs are created using "\code{\link{SetupAnalysisMethods}}"  on a \code{\link{FullDiallelAnalyze}} Object
    
\cr   
# ajLevel:     \cr                  
##  0 if no Aj terms, 1 if        \cr
## MotherModel: MotherModel = 0 if no mother effect to study, \cr
##     0 if no Mother Effect    \cr
##     1 if j dependent           \cr
##     2 if so complicated as to make Thetaij discrete from Thetaji;\cr

\cr
## Cross Epistatsis Models          \cr
##   0 For No Cross Terms             \cr
##   1 For dominance Inbredj model\cr
##   2 For all pooled\cr
##   3 for pool Theta_ij sim N(0, taui^2i * tauj^2)\cr
##   4 for pool Theta_ij sim N(0, taui^2 + tauj^2);\cr
##   5 for All Pooled with Counter Cross terms\cr
##   6 for pool Theta_ij sim N(0, taui^2i * tauj^2), counter cross\cr
##   7 for pool Theta_ij sim N(0, taui^2i + tauj^2), counter cross\cr
##   8 for All Pooled, ij and ji all separate\cr
##   9 for separate ij and ji both N(0, taui^2i * tauj^2)\cr
##   10 for separate ij and ji both N(0, taui^2i + tauj^2)\cr

\cr
## Beta Inbred Level\cr
## 0 No Beta Inbred\cr
## 1 BetaInbred ~ N(0, tauD);\cr
## 2 BetaInbredj ~ N(0, tauDj);  tauDj ~ InvGamma\cr
## 3 BetaInbredj ~ N(BetaInbred, tauDj), tauDj ~InvGamma, BetaInbred ~ tauD; \cr

\cr
## Sex Models\cr
##  0 for No Sex\cr
##  1 for Additive Only Sex\cr
##  2 for Cross Sex aj\cr
##  3 for Cross Sex aj Cross Sex mj\cr
##  4 for Cross Sex aj Cross Sex dj\cr
##  5 for Cross Sex aj Cross Sex mj, Cross taui tauj\cr

\cr
## BetaInbred * Sex \cr
##  0 No BetaInbred * Sex\cr
##  1 Else \cr
##BetaInbredTimesSex = 1;\cr

}
\value{
    Objects
     
     \code{\link{FullDiallelAnalyze}} is the official analysis class.  It contains
      a member \code{AllDiallelObs}  which is a mist of \code{\link{DiallelOb}}'s
      which are model-specific diallel objects.
       
     \item{\code{\link{DiallelOb}}}{R.oo class specific for a given model.  After 
     analysis this contains members \code{raw.chains} and \code{link.chains} which are
     \code{coda} (from pacakge \code{mcmc}) that can be analyzed as Bayesian Gibbs Samples }
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "\code{\link{DiallelOb}}"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{DiallelOb}}, \code{\link{DiallelAnalyzer}} }
\examples{
########################################################################################
## Diallel Example Analyses
##

################################################################################
## Load in R packages
##
##   These R packages must be installed from CRAN to use BayesDiallel to its fullest.
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);


################################################################################
##  First example, examination of "Piximis Mouse Data"
##
##  The data set csv files for inspection of phenotypes for some Diallel data from
## crosses of Collaborative Cross founder strains are within the "Examples"
## directory of this package.  Loading "data(PiximusData)"  automatically loads
## those files into the R terminal.  The dataset is divided into the data file "Piximus.Data"
## the Models File "Examples.Piximus.Models" and the Bayes priors file "Examples.Piximus.tau.Prior.Info".
##
##
##  Length and weight phenotypes can be analyzed using default methods for 
## function "DiallelAnalyzer()"
##

###############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   SaveAFDFile = "SaveAFDBackUpPiximusData.RData");

## Load pregenerated AFD from backup
if (FALSE) {
  library(R.oo);  library(BayesDiallel);
  load("SaveAFDBackUpPiximusData.RData");
}

##List of Elements available to AFD
AFD$ListElements

##List of Functions available to AFD
AFD$ListFunctions;
   
## Get a summary of the Coda Chains for first Diallel Model
summary(AFD$AllDiallelObs[[1]]$raw.chains)
plot(AFD$AllDiallelObs[[1]]$raw.chains);
plot.hpd(AFD$AllDiallelObs[[1]]$raw.chains);

###############################################################################
## Simulate Fake data and experiment with fitting
##############################################################################

TrueModel = c("BSasmvb");
WalkAndReadCrossList(TrueModel)  
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ModelVector = WalkAndReadCrossList(TrueModel);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}


## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)
    
Models =  c(TrueModel, "BSadf3s", "Bvswabsaas");

    
########################################################
##  Call DiallelAnalyze
##
MyDataFileTable = cbind(Y, ListjkInput, SexInputVec, AFDS$FixedEffects, ARandomEffects);
colnames(MyDataFileTable) = c("phenotype", "mother.strain", "father.strain", 
  "is.female", colnames(AFDS$FixedEffects), colnames(ARandomEffects));
AFD = DiallelAnalyzer(DataFile = MyDataFileTable, father.strain="father.strain",
 mother.strain="mother.strain", phenotype="phenotype", is.female="is.female",
   FixedEffects = colnames(AFDS$FixedEffects),
   RandomEffects = ARandomEffects, 
   Models=Models, sigmasq.start = 1;  numChains = 5;  lengthChains=2500,burnin = 1,
   DIC.Only=FALSE,  TauFile = NULL);
   
## List of Elements Available to AFD
AFD$ListElements;
## List of Functions Available to AFD
AFD$ListFunctions;

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb 
                               ## corresponding to our true model

plot(ADO$raw.chains);  ## Plot the chains to understand
summary(ADO$cent.chains);  ## Get Summary information on centered parameters
plot(ADO$cent.chains);  ## Plot the chains to understand
summary(ADO$raw.chains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$raw.chains); ## Using Library_diallel_helper, plot chains.

plot.corr(crosscorr(ADO$raw.chains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
