\name{BayesSpikeDiallel}
\alias{BayesSpikeDiallel}
\alias{BayesSpikeDialel}
\alias{BayesSpikeDiallell}
\alias{BayesSpikeFromDiallel}
\alias{BayesDiallelSpike}
\alias{DoBayesSpike}
\alias{BSSaveDir}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Using BayesSpike with DiallelAnalyzer Function}
\description{
   The \code{\link{DiallelAnalyzer}} wrapper function provides an interface for
     analyzing Diallel features with \link[BayesSpike]{BayesSpike} selection interface.
     
  In short, \link[BayesSpike]{BayesSpike} has the ability to suggest that
   parameters "\eqn{\tau_{k}^2}{tau_k^2}", for groups  
   \eqn{\boldsymbol{\beta}_{\mathbb{J}(k)} \sim N(0, \tau^2_{k})}{Beta(J(k))~N(0,tau_k^2)}
   are a priori with possibility \eqn{\tau^2_{k} = 0}{tau_k^2=0}.
   
   If this is the case, all parameters in a group \eqn{\mathbb{J}(k)}{J(k)} should be zero as well.
   
   By setting  \code{DoBayesSpike} entry to true, as well as  "\code{BSSaveDir}"
   to a user-writable harddrive directory, \link[BayesSpike]{BayesSpike} will
   then be run at the conclusion of \code{\link{DiallelAnalyzer}} runs on
   the original proposed models.   \link[BayesSpike]{BayesSpike} will try to 
   select on a reasonable superset of the models set in the \code{\link{ModelsFile}}.
   
   In addition prior probabilities of activation are supplied through entries to
    \code{PriorProbTau} and \code{PriorProbFixed}.


}
\usage{
  DiallelAnalyzer(data = NULL,
  father.strain="father.strain",
  mother.strain="mother.strain",
  phenotype="phenotype",
  is.female="is_female",
  FixedEffects=NULL,
  RandomEffects=NULL,
  Models=NULL,
  sep=",", na.strings="NA", 
  sigmasq.start = 1, numChains =3, lengthChains = 2000,
  Verbose = TRUE,
  burnin = 1, thin = 1, DIC.Only = FALSE,
  TauFile = NULL,  
  DoBayesSpike=FALSE 
  PriorProbTau = NULL,
  PriorProbFixed = NULL, 
  BSSaveDir = NULL,);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  We describe some \link[BayesSpike]{BayesSpike} relevant flags to set for 
  \code{\link{DiallelAnalyzer}} function.
  \item{DoBayesSpike}{Logical Flag, When set to TRUE, 
    BayesSpike will be attempted at end of \code{\link{DiallelAnalyzer}} run.}
  \item{BSSaveDir}{A string for a file save directory.  A user-accessible 
    Save directory on HardDrive space
   is necessary to use \code{\link[BayesSpike]{BayesSpike}} package.
   See \code{\link[BayesSpike]{SetupSaveFile}} for a description of the
   BayesSpike save-directory needs.}
  \item{PriorProbFixed}{Prior Probabilities of whether Fixed parameters should be non-zero.  Note that
  setting \code{tauPriorFile} in \code{\link{DiallelAnalyzer}} to have a row describing
   prior content works as well.}
  \item{PriorProbTau}{Prior Probabilities of whether Random Groups parameters should be non-zero.  Note that
  setting \code{tauPriorFile} in \code{\link{DiallelAnalyzer}} to have a row describing
   prior content works as well.}
    
Old, well known arguments:
  \item{data}{Data frame containing data, every row is a diallel subject, else a Filename where file is separated by "sep"}
  \item{phenotype}{Name of column in "DataFile" that is the dependent phenotype}
  \item{father.strain}{Name of column in "DataFile" that is the father strain indicator, could be numeric or character, reduced to a factor}
  \item{mother.strain}{Name of column in "DataFile" that is the mother strain indicator, could be numeric or character, reduced to a factor}
  \item{is.female}{Name of column in "DataFile" that records gender of the subject}    
  \item{FixedEffects}{Column or Columns of Data (in DataFile) with additional fixed effects in recorded}
  \item{RandomEffects}{Column or Columns of Data (in DataFile) with additional Random Effects (factors) if recorded}
  \item{Models}{Character vector, or matrix of model values describing models or superset of models of interest}
  \item{sep}{ Separator of columns if DataFile is a file to be read in}
  \item{na.strings}{Character set representing "NA" value for unknowns in the table}
  \item{Sigma.start}{Staring Value of Noise to be used in chains (will be learned in Gibbs Sampling}
  \item{lengthChains}{ Length of Gibbs Sampler Chains}
  \item{numChains}{Number of Gibbs Sampler Chains}
  \item{Verbose}{Give feedback during the operation?}
  \item{burnin.start}{Automatically kill chains starting up untill burnin.start}
  \item{burnin.end}{Kill chains at end of burnin.end, default length of chains}
  \item{thin}{Thin chains by deleting rows every thin amount}
  \item{DIC.only}{Report only the DIC of the models tested, delete chains as run for more memory}
  \item{TauFile}{An Optional file of Mean and degree of freedom for inverse-gamma priors 
    for the various groups in the algorithm (including additional Random Effects}
}
\details{
  \code{\link{FullDiallel}} objects come in two types, \code{\link{FullDiallelSimulate}},
    and \code{\link{FullDiallelAnalyze}}.  Analyses will be done with \code{FullDiallelAnalyze} objects.

  \code{\link{DiallelOb}} has several default functions including "Copy" and "finalize" for 
  deleting.  
  
  However, the \code{\link[BayesSpike]{BayesSpike}} member will be attached as element "\code{FullDiallelAnalyzeObject$BS}".
  It will be a \link[Rcpp]{Rcpp} module object specifically of type \code{\link[BayesSpike]{BayesSpikeCpp}}.
  
  Remember that \code{BSSaveDir} must be set to a user accessible directory.  
    Binary files will be saved in this directory following the work.  Potentially
    these files can be reread after a run, however the user interface to these
    is currently in development.  It is not possible to \code{\link[base]{save}}
    the \link[Rcpp]{Rcpp} object by itself after a run.  Instead, saving the 
    \code{$CodaList} elements manually should provide a recording of the 
    \code{\link[BayesSpike]{BayesSpikeRegression}} output for later analyses.  
  Analysis functions for \code{$CodaList} element are the same as all
  "Coda" package 
  \code{\link[coda]{mcmc}}  tables, including \code{summary}, \code{plot},
  \code{plot.hpd}, etc.
  
  Example Datasets are found in \code{\link{PiximusData}}, \code{\link{AdviaData}}, \code{\link{ChemData}}.
}
\value{
    Object returned from \code{\link{DiallelAnalyzer}} is a  \code{\link{FullDiallelAnalyze}} object.
    
    If \code{DoBayesSpike} was run, then the returned  \code{\link{FullDiallelAnalyze}} contains a member 
    element "\code{BS}" (aliased to \code{BSAFD}) which is a  \link[Rcpp]{Rcpp} type object,
    specifically of type \link{BayesSpike}{BayesSpikeCpp}  which contains
    results of the BayesSpikeRegression.    A list of all accessible elements of 
    \link{BayesSpike}{BayesSpikeCpp} are available on that page.
    
    For Diallel purposes the \code{BS$CodaList} element is the  Coda \link{coda}{mcmc} table which is
     valuable for plotting and studying the features of the chain.  
     \code{summary} functions exist as well.  
     Note that the Coda \link{coda}{mcmc} table for \link[BayesSpike]{BayesSpike} 
     potentially has more members than just (\eqn{\boldsymbol{\beta}}{Beta},
     \eqn{\boldsymbol{\tau^2}}{tau^2} \eqn{\sigma^2}{sigma^2}) draws.
     Supposing that features of \link[BayesSpike]{BayesSpike} input vector
       \link[BayesSpike]{DoRecord} have been set it will report Rao-Blackwellized
       Probabilities for fixed parameter \eqn{\beta_{j}}{Beta_j} as well
       as group parameter \eqn{\tau^2_{k}}{tau_k^2} non-zero probabilities.  

}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "\code{\link[BayesSpike]{BayesSpike}}" requires package \code{\link[Rcpp]{Rcpp}}
  to operate \code{\link{BayesSpike}{BayesSpikeCpp}} output class.  Also
  \code{\link[BayesSpike]{BayesSpike}} objects come with their own "R5"
  "\code{\link[methods]{ReferenceClasses}}"" reference class.  This is a midway 
  object that preserves data before conversion to \code{\link[Rcpp]{Rcpp}} module form.
   Pass by reference still results in certain copies within and of the
   \code{\link[BayesSpike]{BayesSpikeR5}} class, so most memory preservation
   is done within \code{\link[Rcpp]{Rcpp}}.  
  
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{DiallelOb}}, \code{\link{DiallelAnalyzer}},
  \code{\link[BayesSpike]{BayesSpikeFromAutoDiallel}},
  \code{\link[BayesSpike]{BayesSpike}}, 
  \code{\link[BayesSpike]{BayesSpikeRegression}},
  \code{\link[BayesSpike]{BayesSpikeCpp}},
  \code{\link[BayesSpike]{BayesSpikeR5}},
  \code{\link[BayesSpike]{BayesSpike}},
  \code{\link[BayesSpike]{BayesSpikeFromAutoDiallel}}
  
  }
\examples{
library(BayesDiallel);
###############################################################################
## Running BayesSpike on First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
##########################################################################
##  BayesSpike Selection uses Prior distributions to select a valid model from
##   subset of other models.  "tauPriorFile" can have an extra row "Prob"
##   with probabilities, if one wants to select certain groups with different
##   prior probabilities.  Default "Prob" will be .5
##
##  See library(BayesSpike) for more details on this selection process.
##
## Experimental, might not work if you don't have directory write access
## Experimental, might not work if you don't have directory write access
if (length(list.files("C:")) > 0) {
  try(dir.create("C:/Stat"));  try(dir.create("C:/Stat/BayesSpikeSaves"));
  BSSaveDir = "C:/Stat/BayesSpikeSaves"
} else if (length(list.files("~/Documents")) > 0) {
  try(dir.create("~/Documents/Stat"));  try(dir.create("~/Documents/Stat/BayesSpikeSaves"));
  BSSaveDir = "~/Documents/Stat/BayesSpikeSaves"
} else if (length(list.files("~/")) > 0) {
 try(dir.create("~/Stat"));  try(dir.create("~/Stat/BayesSpikeSaves"));
  BSSaveDir = "~/Stat/BayesSpikeSaves"
}

MyProcTime = tSeq(proc.time()[1]);
BSSaveDir  = paste(BSSaveDir, MyProcTime, sep="");  ## add a time stamp to analysis

AFD2 = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", 
   is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[1:2], sigmasq.start = 1,  
   numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   DoBayesSpike = TRUE, BSSaveDir = BSSaveDir);
   
BSAFD <- AFD2$BSAFD;
BSAFD$MIP;


  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
