\name{CombineDiallels}
\alias{CombineDiallesl}
\alias{CombineDiallels}
\alias{combinediallels}
\alias{joindiallels}
\alias{join.diallels}
\alias{DiallelCombine}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Create new difference object.}
\description{
\code{\link{CombineDiallels}} is a function of a \code{\link{FullDiallelAnalyze}} object
 and a second object.  
 
 \code{\link{Diff2AFD}} is a function of two \code{\link{FullDiallelAnalyze}} objects.

 One must have fit both models and hope they are of same structure.  The Diff of
 chains however, may be informative about treatment interactions of Betas.
 This is why one would want to generate a Diff object.
  
}
\usage{
Diff2AFD(AFD1,AFD2);

AFD1$DiffAFD(AFD2);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{AFD1}{One fitted \code{\link{FullDiallelAnalyze}}  object for one dataset}
  \item{AFD2}{Second fitted \code{\link{FullDiallelAnalyze}} object for possibly different dataset, but same models.}
 
}
\details{
  Good functions to take of the output is \code{summary} and
    \code{\link{PlotHPD}}.
  

}
\value{
  A difference \code{\link{FullDiallelAnalyze}} object.
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{FullDiallel}}, \code{\link{DiallelAnalyzer}} }
\examples{
library(AlanDirectories);
SaveDir <- paste(DropHome, "//PsyDiallelSims//PracticeCombine", sep="");
BSSaveDir <- paste(SaveDir, "//BS//", sep="");
dir.create(SaveDir, showWarnings=FALSE, recursive=TRUE);
dir.create(BSSaveDir, showWarnings=FALSE, recursive=TRUE);
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
##############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD1 = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="TotalTissueMass", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[1:2], sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = paste(SaveDir, "//SaveAFDBackAFD1.RData", sep=""), 
   DoBayesSpike = TRUE, BSSaveDir=paste(BSSaveDir, "/D1", sep=""));  
   save(AFD1, paste(SaveDir, "//SaveAFDBackAFD1.RData", sep="")); 
   try(AFD1$BSAFD$Save());  
   ##save(AFD1$BSAFD, paste(SaveDir, "//SaveBSAFDBackAFD1.RData", sep=""));                 
AFD2 = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="LeanTissueMass", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[1:2], sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = paste(SaveDir, "//SaveAFDBackAFD2.RData", sep=""), 
   DoBayesSpike = TRUE, BSSaveDir=paste(BSSaveDir, "/D2", sep="") );    
save(AFD2, paste(SaveDir, "//SaveAFDBackAFD2.RData", sep=""));  
try(AFD2$BSAFD$Save()); 
##save(AFD2$BSAFD, paste(SaveDir, "//SaveBSAFDBackAFD2.RData", sep="");               
AFD3 = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="BoneMineralContent", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[1:2], sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = paste(SaveDir, "//SaveAFDBackAFD3.RData", sep=""), 
   DoBayesSpike = TRUE, BSSaveDir=paste(BSSaveDir, "/D3", sep="") );  
   try(AFD3$BSAFD$Save());  
save(AFD3, paste(SaveDir, "//SaveAFDBackAFD3.RData", sep=""));  
##save(AFD3$BSAFD, paste(SaveDir, "//SaveBSAFDBackAFD3.RData", sep="")); 
  
## Load pre-generated AFD from backup
if (FALSE) {

  library(R.oo);  library(BayesDiallel);
  library(AlanDirectories);
  SaveDir <- paste(DropHome, "//PsyDiallelSims//PracticeCombine", sep="");
  BSSaveDir <- paste(SaveDir, "//BS//", sep="");
  library(BayesSpike);
  load(paste(SaveDir, "//SaveAFDBackAFD1.RData", sep=""));
  AFD1 <- saveLoadReference;
  load(paste(SaveDir, "//BS//D1//FinalOutputSave//PartSaveListTBSR5.RData", sep=""));
  NewBSObject <- ReLoad(MyList)
  AFD1$.BS <- NewBSObject;
  
 load(paste(SaveDir, "//SaveAFDBackAFD2.RData", sep=""));
  AFD2 <- saveLoadReference;
  load(paste(SaveDir, "//BS//D2//FinalOutputSave//PartSaveListTBSR5.RData", sep=""));
  NewBSObject <- ReLoad(MyList)
  AFD2$.BS <- NewBSObject;

  load(paste(SaveDir, "//SaveAFDBackAFD3.RData", sep=""));
  AFD3 <- saveLoadReference;
  load(paste(SaveDir, "//BS//D3//FinalOutputSave//PartSaveListTBSR5.RData", sep=""));
  NewBSObject <- ReLoad(MyList)
  AFD3$.BS <- NewBSObject;
  
}

## Take the Diff!
AFDCombine <- CombineDiallels(AFD1,AFD2, AFD3);

## Alternatively one could use R lists 
##   if one had already combined all AFD objects into a list
AFDList <- list(AFD1=AFD1, AFD2=AFD2, AFD3=AFD3);
AFDCombine <- CombineDiallels(AFDList);
  
  

## Summary of significant differences!
summary(AFDCombine$AllDiallelObs[[1]]$cent.chains);
 
## Now make those HPD Plots of the Difference!
AFDCombine$AllDiallelObs[[1]]$PlotHPD();
AFDCombine$AllDiallelObs[[1]]$PlotHPD(wanted = c("aj", "Gender:aj"), name.margin = 7, title.margin = 5,
  xlab="Confidence intervals");
AFDCombine$AllDiallelObs[[1]]$PlotHPD(wanted = c("aj", "Gender:aj"), name.margin = 7, title.margin = 5,
  xlab="Confidence intervals", xlims=c(-2,2) );
  
  library(AlanDirectories);  SetSaveHomes();
pdf(paste(DropHome, "//RDiallel/PicturePractice/APracticeHPDPlot.pdf", sep=""),
  width = 10.5, height = 7.5); 
AFDCombine$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE,
  UseMapChainNamesHPD = TRUE,
  xlab="Confidence intervals", EvenXlims=TRUE, DoEvensPlot=TRUE, DoMu = FALSE);
dev.off();

AFDCombine$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE,
  UseMapChainNames1 = TRUE,
  xlab="Confidence intervals", EvenXlims=TRUE);

## Plot with DefaultWanted1 = TRUE, be sure to plot "Treatment Effect"
AFDCombine$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE, DoTreat = TRUE)

## "d^2" and "h^2" heritability measures
AFDCombine$AllDiallelObs[[1]]$PosteriorHSqTable;

AFDCombine$AllDiallelObs[[1]]$PosteriorDSqTable;

## A Sub Table of D^2 summarizes the DSq inheritance effects into
##   Additive and Broad sense effects.
AFDCombine$AllDiallelObs[[1]]$PosteriorDSqSubTable;

## if the AFD Objects had AFD$BSAFD set [if you ran BayesSpike]
MIP <- AFDCombine$BSAFD$MIP
AFDBSChains <- AFDCombine$BSAFD$CodaList;
ColnamesBSChains <- AFD$Combine$BSAFD$OtherCodaNames;
    

library(BayesDiallel); library(BayesSpike); library(AlanDirectories);
AlanDirectories:::SetSaveHomes();
SaveDir <- paste(DropHome, "//PsyDiallelSims//PracticeCombine", sep="");
BSSaveDir <- paste(SaveDir, "//BS//", sep="");
dir.create(SaveDir, showWarnings=FALSE, recursive=TRUE);
dir.create(BSSaveDir, showWarnings=FALSE, recursive=TRUE);
##############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD1 = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="TotalTissueMass", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[1:2], sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE, DoFirstCenter=TRUE,
   SaveAFDFile = paste(SaveDir, "//SaveAFDCenterBackAFD1.RData", sep=""), 
   DoBayesSpike = TRUE, BSSaveDir=paste(BSSaveDir, "/DC1", sep=""));  
   save(AFD1, paste(SaveDir, "//SaveAFDCenterBackAFD1.RData", sep="")); 
   try(AFD1$BSAFD$Save());                  
AFD2 = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="LeanTissueMass", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[1:2], sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = paste(SaveDir, "//SaveAFDCenterBackAFD2.RData", sep=""), 
   DoBayesSpike = TRUE, BSSaveDir=paste(BSSaveDir, "/DC2", sep="") );    
save(AFD2, paste(SaveDir, "//SaveAFDCenterBackAFD2.RData", sep=""));  
try(AFD2$BSAFD$Save()); 
##save(AFD2$BSAFD, paste(SaveDir, "//SaveBSAFDBackAFD2.RData", sep="");               
AFD3 = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="BoneMineralContent", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[1:2], sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = paste(SaveDir, "//SaveAFDCenterBackAFD3.RData", sep=""), 
   DoBayesSpike = TRUE, BSSaveDir=paste(BSSaveDir, "/DC3", sep="") );  
   try(AFD3$BSAFD$Save());  
save(AFD3, paste(SaveDir, "//SaveAFDCenterBackAFD3.RData", sep=""));  
##save(AFD3$BSAFD, paste(SaveDir, "//SaveBSAFDBackAFD3.RData", sep="")); 
  
## Load pre-generated AFD from backup
if (FALSE) {

  library(R.oo);  library(BayesDiallel);
  library(AlanDirectories);
  SaveDir <- paste(DropHome, "//PsyDiallelSims//PracticeCombine", sep="");
  BSSaveDir <- paste(SaveDir, "//BS//", sep="");
  library(BayesSpike);
  load(paste(SaveDir, "//SaveAFDCenterBackAFD1.RData", sep=""));
  AFD1 <- saveLoadReference;
  load(paste(SaveDir, "//BS//DC1//FinalOutputSave//PartSaveListTBSR5.RData", sep=""));
  NewBSObject <- ReLoad(MyList)
  AFD1$.BS <- NewBSObject;
  
 load(paste(SaveDir, "//SaveAFDCenterBackAFD2.RData", sep=""));
  AFD2 <- saveLoadReference;
  load(paste(SaveDir, "//BS//DC2//FinalOutputSave//PartSaveListTBSR5.RData", sep=""));
  NewBSObject <- ReLoad(MyList)
  AFD2$.BS <- NewBSObject;

  load(paste(SaveDir, "//SaveAFDCenterBackAFD3.RData", sep=""));
  AFD3 <- saveLoadReference;
  load(paste(SaveDir, "//BS//DC3//FinalOutputSave//PartSaveListTBSR5.RData", sep=""));
  NewBSObject <- ReLoad(MyList)
  AFD3$.BS <- NewBSObject;
  
}

## Take the Diff!
AFDCombine <- CombineDiallels(AFD1,AFD2, AFD3);    
  
  
AFDCombine$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE,
  UseMapChainNamesHPD = TRUE, NoSex=TRUE, DoEvensPlot=TRUE);
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
