\name{DiallelAnalyzer}
\alias{DiallelAnalyzer}
\alias{DiallelAnalyze}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Diallel Analyzer, One stop function for studying a Diallel Dataset}
\description{
  This wrapper function performs most work necessary to automatically analyze
    a Diallel Dataset.  Officially, it generates an S3 "\code{\link{FullDiallelAnalyze}}" class,
    initializes the models suggested by input, and then performs all the Gibbs
    steps, returning the \code{\link{FullDiallelAnalyze}} object which has gibbs chains properly
    assigned to "My\code{\link{FullDiallelAnalyze}}Ob$All\code{\link{DiallelOb}}s[[ii]]$cent.chains"
    and the "My\code{\link{FullDiallelAnalyze}}Ob$All\code{\link{DiallelOb}}s[[ii]]$CodaChains" is a coda mcmc
    object with the chain output.
}
\usage{
  DiallelAnalyzer(DataFile = NULL,
  father.strain="father.strain",
  mother.strain="mother.strain",
  phenotype="phenotype",
  is.female="is.female",
  SexVector=NULL,
  FixedEffects=NULL, nameFixedEffects=NULL,
  LogTransform = FALSE, SqrtTransform = FALSE,
  RandomEffects=NULL, nameRandomEffects=NULL,
  Models=NULL,
  sep=",", na.strings="NA", 
  sigmasq.start = 1, numChains =3, lengthChains = 2000,
  Verbose = TRUE,
  burnin.start = 1, burnin.end = -1, burnin.thin = 1, DIC.Only = FALSE,
  TauFile = NULL, DoBayesSpike=FALSE, BSSaveDir ="",SaveAFDFile="",
  LogTransform=FALSE, SqrtTransform=FALSE,
  ProbFemaleUnknownSex=NULL, ListUnknownSex=NULL, NoChainsUnknownSex=NULL,
  MissingYIndices = NULL,  NoChainsImputeY=FALSE,
  LowCensorBound = NULL, UpCensorBound = NULL, DoGelman=FALSE);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{DataFile}{Columns from a datalist, every row is a diallel subject, else a DataFile in a directory to be read in}
  \item{phenotype}{Name of column in "DataFile" that is the dependent phenotype}
  \item{father.strain}{Name of column in "DataFile" that is the father strain indicator, could be numeric or character, reduced to a factor}
  \item{mother.strain}{Name of column in "DataFile" that is the mother strain indicator, could be numeric or character, reduced to a factor}
  \item{is.female}{Name of column in "DataFile" that records gender of the subject}    
  \item{FixedEffects}{Column or Columns of Data (in DataFile) with additional fixed effects in recorded}
  \item{RandomEffects}{Column or Columns of Data (in DataFile) with additional Random Effects (factors) if recorded}
  \item{nameFixedEffects}{Name to give columns of Fixed Effects to be stored for column name}
  \item{nameRandomEffects}{Name to be given to Random Effects Groups}
  \item{Models}{Character vector, or matrix of model values describing models or superset of models of interest}
  \item{sep}{ Separator of columns if DataFile is a file to be read in}
  \item{na.strings}{Character set representing "NA" value for unknowns in the table}
  \item{sigmasq.start}{Staring Value of Noise to be used in chains (will be learned in Gibbs Sampling}
  \item{lengthChains}{ Length of Gibbs Sampler Chains}
  \item{LogTransform}{Binary indicator whether to log Tranform phenotype data before analysis}
  \item{SqrtTransform}{Binary indicator whether to square root Tranform phenotype data before analysis}
  \item{numChains}{Number of Gibbs Sampler Chains}
  \item{Verbose}{Give feedback during the operation?}
  \item{burnin.start}{Automatically kill chains starting up untill burnin.start}
  \item{burnin.end}{Kill chains at end of burnin.end, default length of chains}
  \item{burnin.thin}{Thin chains by deleting rows every burnin.thin amount}
  \item{DIC.only}{Report only the DIC of the models tested, delete chains as run for more memory}
  \item{TauFile}{An Optional file of Mean and degree of freedom for inverse-gamma priors 
    for the various groups in the algorithm (including additional Random Effects}
  \item{DoBayesSpike}{Flag set to whether to do BayesSpike Selection in addition to individual models, BSSaveDir must be set!}
  \item{BSSaveDir}{A directory to store samples on the hard-drive for BayesSpike Selection.  BayesSpike will try to identify a library to save to,
    but to be safe, please provide a directory which might be interesting.}
  \item{SaveAFDFile}{A file to save finished product.  In this RData file, the \code{\link{FullDiallelAnalyze}} object is named "AFD"}
  \item{Wanted}{See method \code{\link{SetupAnalysisMethods}} A list a subset of Beta variables 
  in case a subset is all one desires to save to Gibbs chains}
  \item{Extractor}{See method \code{\link{SetupAnalysisMethods}}, A function containing extractor functions to steal variables if a subset or
    function of subset of data is desired.}
  \item{ListUnknownSex}{See method \code{\link{SetupUnknownSex}} A list of specimen numbers for which sex is unknown.
    Alias is \code{ListUnknownGender}.}
  \item{ProbFemaleUnknownSex}{See method \code{\link{SetupUnknownSex}} A vector of prior probabilities that each 
    specimen with unknown gender is male or female.  Alias is \code{ProbUnknownGender}}
  \item{NoChainsUnknownSex}{ A flag on whether to save \code{\link[coda]{mcmc}} chains for the sex imputation. 
    Alias is \code{NoChainsUnknownGender}}
  \item{LogTransform}{A flag whether phenotype should be log transformed before analysis}
  \item{SqrtTransform}{A flag whether phenotype should be sqrt transformed before analysis}
  \item{MissingYIndices}{Indices of Y that can be considered Censored or missing.  If any Y are NA, then this
    will be automatically generated}
  \item{LowCensorBound}{Somewhat of a misnomer.  If Data that is "NA" or missing in MissingYIters is considered
    "Censored", in that it is generated NA if the data is above the LowCensorBound, then Imputed Missing Y
    will only be imputed as values GREATER than LowCensorBound.  If LowCensorBound is NULL, there is no lower
    boundary.}
  \item{UpCensorBound}{Somewhat of a misnomer.  If Data that is "NA" or missing in MissingYIters is considered
    "Censored", in that it is generated NA if the data is below the UpCensorBound, then Imputed Missing Y
    will only be imputed as values LESS than UpCensorBound.  If UpCensorBound is NULL, there is no upper boundary.}
  \item{NoChainsImputeY}{Set to TRUE if one does not want to collect coda values for imputed Y.  Imputed Y 
    values will be sored in \code{\link{DiallelOb}} piece "ImputedYChains"}
  \item{DoGelman}{Logical Flag whether to use Gelman-Rubin analysis to judge convergence/mixing of chains and
     whether to double cain length and run again.  For problematic datasets, the BayesDiallel procedure
     may decide that many chain results are needed.  By Setting DoGelman = FALSE, the algorithm will
     stop after the initial first chains are filled in.}
  
  For information on using \code{\link{DiallelAnalyzer}} with \link[BayesSpike]{BayesSpike}
  see entry for \link{BayesSpikeDiallel}  and \link[BayesSpike]{BayesSpikeFromAutoDiallel}.
}
\details{
  \code{\link{DiallelOb}} has several default functions including "copy" and "finalize" for 
  deleting.  
  
  Hidden members .numj (number of diallel strains), .ajModel, .MotherModel,
    .CrossModel develop from model cross levels
    .SexModel, .BetaInbredLevel, .BetaHybirdLevelTimesSex
    
  DiallelObs are created using "\code{\link{SetupAnalysisMethods}}"  on a \code{\link{FullDiallelAnalyze}} Object
  
  Note, DiallelAnalyzer will save output after running chains to "SaveAFDFile" if you set it.  Then it is easy to reload analysis using \code{\link{load}}
   (But you must \code{\link{library}} in \code{\link{R.oo}} and \code{\link{BayesDiallel}} before loading in SaveAFDFile).
  
#### The following is unnecessary documentation of internal model coding structure.
# ajLevel:     \cr                  
##  0 if no Aj terms, 1 if        \cr
## MotherModel: MotherModel = 0 if no mother effect to study, \cr
##     0 if no Mother Effect    \cr
##     1 if j dependent           \cr
##     2 if so complicated as to make Thetaij discrete from Thetaji;\cr

\cr
## Cross Epistatsis Models          \cr
##   0 For No Cross Terms             \cr
##   1 For dominance Hybridj model\cr
##   2 For all pooled\cr
##   3 for pool Theta_ij sim N(0, taui^2i * tauj^2)\cr
##   4 for pool Theta_ij sim N(0, taui^2 + tauj^2);\cr
##   5 for All Pooled with Counter Cross terms\cr
##   6 for pool Theta_ij sim N(0, taui^2i * tauj^2), counter cross\cr
##   7 for pool Theta_ij sim N(0, taui^2i + tauj^2), counter cross\cr
##   8 for All Pooled, ij and ji all separate\cr
##   9 for separate ij and ji both N(0, taui^2i * tauj^2)\cr
##   10 for separate ij and ji both N(0, taui^2i + tauj^2)\cr

\cr
## Beta Hybrid Level\cr
## 0 No Beta Hybrid\cr
## 1 BetaHybrid ~ N(0, tauD);\cr
## 2 BetaHybridj ~ N(0, tauDj);  tauDj ~ InvGamma\cr
## 3 BetaHybridj ~ N(BetaHybrid, tauDj), tauDj ~InvGamma, BetaHybrid ~ tauD; \cr

\cr
## Sex Models\cr
##  0 for No Sex\cr
##  1 for Additive Only Sex\cr
##  2 for Cross Sex aj\cr
##  3 for Cross Sex aj Cross Sex mj\cr
##  4 for Cross Sex aj Cross Sex dj\cr
##  5 for Cross Sex aj Cross Sex mj, Cross taui tauj\cr

\cr
## BetaHybrid * Sex \cr
##  0 No BetaHybrid * Sex\cr
##  1 Else \cr
##BetaInbredTimesSex = 1;\cr

}
\value{
    Object  
     \item{FullDiallelAnalyze}{This is Class object type that is returned, 
       See "\code{\link{FullDiallelAnalyze}}" for more help}
     \item{AllDiallelObs}{This is a list member of return object.  It is a list of 
       "\code{\link{DiallelOb}}"  R.oo classes.
     
       See "DiallelOb" for members of DiallelOb.  "cent.chains"  and "raw.chains"
       are the Coda objects representing mcmc output.}
     
       
       
    AFD$strain.map records the strain mappings from the analysis.
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{ \code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{DiallelOb}}, 
  \code{\link{BayesSpikeDiallel}},
  \code{\link[BayesSpike]{BayesSpike}},
  \code{\link[BayesSpike]{BayesSpikeFromAutoDiallel}}}
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);

###############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
if (FALSE) {
  data = Piximus.Data;  father.strain="father.strain.name";
  mother.strain = "mother.strain.name"; phenotype="MouseWeight";
  is.female = "is.female";  sep=""; FixedEffects=NULL; RandomEffects=NULL;
  Models = Example.Piximus.Models[c(1,3,5)]; sigmasq.start = 1;
  numChains = 2;  lengthChains = 1000; burnin = 1;  DIC.Only = FALSE;
  tauPriorFile = Example.Piximus.tau.Prior.Info;  LogTransform=TRUE;
  phenotype.name="";  DoGelman = TRUE;
}
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[c(1,3,5)], sigmasq.start = 1,  numChains = 2,  lengthChains=1000,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   SaveAFDFile = "SaveAFDBackUpPiximusDataShort.RData",
   LogTransform=TRUE, phenotype.name = "", DoGelman = TRUE);
   
   ###### Run the Algorithm->
AFDCent = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models[c(1,3,5)], sigmasq.start = 1,  numChains = 2,  lengthChains=1000,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   SaveAFDFile = "SaveAFDBackUpCenterPiximusDataShort.RData",
   LogTransform=TRUE, phenotype.name = "", DoGelman = TRUE, DoFirstCenter=TRUE);

## If SaveAFDFile saved a copy of AFD
if (FALSE) {
  library(R.oo);  library(BayesDiallel);
  load("SaveAFDBackUpPiximusDataShort.RData");
  AFD1 = AFD;
  
  library(R.oo);  library(BayesDiallel);
  load("SaveAFDBackUpCenterPiximusDataShort.RData");
  AFDCent = AFD;
  
  AFD = AFD1;
  
}
AFD$ListElements;  ## List of Elements available to AFD
AFD$ListFunctions; ## List of Functions available to AFD  

AO = AFD$AllDiallelObs[[1]];
AO$ListElements  ## List of Elements available to AFD

## Chains are the first version, they are preshrunk towards group means.  You should
##   often use these by default;
summary(AO$cent.chains);   ## summary of the chains;
plot(AO$cent.chains);      ## Use Coda MCMC to plot the many member of chains consider "ask"/"wanted"
plot.hpd(AO$cent.chains);  ## Plot Hpd intervals for members of "chains" object.  Consider "ask"/"wanted"
summary(AO$raw.chains);   ## summary of the chains;
plot(AO$raw.chains);      ## Use Coda MCMC to plot the many member of chains consider "ask"/"wanted"
plot.hpd(AO$raw.chains);  ## Plot Hpd intervals for members of "chains" object.  Consider "ask"/"wanted"
  
## raw.chains are the "unshrunk version"
## Get a summary of the Coda Chains for first Diallel Model
summary(AFD$AllDiallelObs[[1]]$raw.chains)
plot(AFD$AllDiallelObs[[1]]$raw.chains);
plot.hpd(AFD$AllDiallelObs[[1]]$raw.chains);


## Plot Females, observed versus fitted
AFD$AllDiallelObs[[1]]$TwoDiallelPlot(FemalePlot = TRUE, MaleAgainstFemale = FALSE);

#################################################################################
##  HSq and RSq
##
##   HSq is our future heritability metric, based upon estimated tau^2 parameters,
##    it assumes independence of  future diallels.
##
##   RSq relates the amount of variation in Y explained by a covariate.  Note, that
##    because of correlation between terms, sum of RSq terms can be more or less than
##    total variation of Y.  
##
## Give a table summarizing beliefs in heritability ratios for future independent diallels.
##  expresses residual variance in terms of 
AFD$AllDiallelObs[[1]]$PosteriorHSqTable

## Inner method that calculates that HSq
OtherPosteriorChains = AFD$AllDiallelObs[[1]]$PosteriorHSq(UseFixed=TRUE, ProperRatio=TRUE,
      AFD = AFD, startiter = AFD$burin, thin = AFD$thin)
      
## Now look at RSq as a different metric
summary(AFD$AllDiallelObs[[1]]$PosteriorRSqChains);

############################################################################
## DSq
##   DSq exists in the realm between H and R
##   It is a quantity trying to explain "Beta Effect" variability
##     but it operates only on fake data, not on real data,
##     where this fake data is generated from a theoretical "One complete Diallel"
AFD$AllDiallelObs[[1]]$PosteriorDSqTable


#######################################################################
##  PlotStrawPlot
##
##    Plots a Straw Plot of effects, this shows fixed estimates and how they
##     influence displacements of inbreds.
##
## PlotStrawPlot illustrates effects
AFD$AllDiallelObs[[1]]$PlotStrawPlot();

##########################################################################
##  BayesSpike Selection uses Prior distributions to select a valid model from
##   subset of other models.  "tauPriorFile" can have an extra row "Prob"
##   with probabilities, if one wants to select certain groups with different
##   prior probabilities.  Default "Prob" will be .5
##
##  See library(BayesSpike) for more details on this selection process.
##
## Experimental, might not work if you don't have directory write access
if (length(list.files("C:")) > 0) {
  try(dir.create("C:/Stat"));  try(dir.create("C:/Stat/BayesSpikeSaves"));
  BSSaveDir = "C:/Stat/BayesSpikeSaves"
} else if (length(list.files("~/Documents")) > 0) {
  try(dir.create("~/Documents/Stat"));  
  try(dir.create("~/Documents/Stat/BayesSpikeSaves"));
  BSSaveDir = "~/Documents/Stat/BayesSpikeSaves"
} else if (length(list.files("~/")) > 0) {
 try(dir.create("~/Stat"));  try(dir.create("~/Stat/BayesSpikeSaves"));
  BSSaveDir = "~/Stat/BayesSpikeSaves"
}
AFD2 = DiallelAnalyzer(data = Piximus.Data, 
  father.strain="father.strain.name", 
  mother.strain="mother.strain.name", 
  phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  
   numChains = 5,  lengthChains=2500, burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   DoBayesSpike = TRUE, BSSaveDir = BSSaveDir,
   SaveAFDFile ="", LogTransform=TRUE);
   
## Get a summary of the Coda Chains for first Diallel Model
summary(AFD2$BSAFD$CodaList)
plot(AFD2$BSAFD$CodaList);
plot.hpd(AFD2$BSAFD$CodaList);

#########################################################################
## Simulate Fake data and experiment with fitting
#########################################################################
TrueModel = c("BSasmvb");
WalkAndReadCrossList(TrueModel)  
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ModelVector = WalkAndReadCrossList(TrueModel);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4]; ## 09062014 - Replacing BetaHybridLevel with BetaInbredLevel
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = cbind(rnorm(WantLen), .5* rnorm(WantLen));
   colnames(InFixedEffects) = c("FEffect1", "FEffect2")
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj,
 InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, 
 NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}


## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, 
  CrossModel = CrossModel, SexModel = SexModel, 
  BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and 
##   observed data Y      
SimValues(AFDS, Sigma = 1);

#########################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data
###   to put in an  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
##FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we
##   would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)
    
Models =  c(TrueModel, "BSadf3s", "Bvswabsaas");

    
########################################################
##  Assemble a data table.
##
MyDataFileTable = cbind(Y, ListjkInput, SexInputVec,
  AFDS$FixedEffects, ARandomEffects);
cnFixedEffects = colnames(AFDS$FixedEffects);
cnRandomEffects = colnames(ARandomEffects);
if (length(ARandomEffects) > 0 && 
  is.null(cnRandomEffects)) {
  cnRandomEffects = "RandomEffect:1";
}
colnames(MyDataFileTable) = c("phenotype", 
  "mother.strain", "father.strain", 
  "is.female", colnames(AFDS$FixedEffects), 
  cnRandomEffects);
if (FALSE) {

data = MyDataFileTable; 
 father.strain="father.strain"; mother.strain="mother.strain"; 
   phenotype="phenotype"; is.female="is.female";
   SexVector="sex"; FixedEffects = colnames(AFDS$FixedEffects);
   RandomEffects = cnRandomEffects; 
   Models=Models; sigmasq.start = 1;  numChains = 5;
   lengthChains=2500;burnin.start = 1;
   DIC.Only=FALSE;  tauPriorFile = NULL; DoBayesSpike = FALSE; 
   Verbose = 3
}
  
########################################################
##  Call DiallelAnalyze
##  
AFD = DiallelAnalyzer(data = MyDataFileTable, DataFile = MyDataFileTable, 
 father.strain="father.strain", mother.strain="mother.strain", 
   phenotype="phenotype", is.female="is.female",
   SexVector="sex", FixedEffects = colnames(AFDS$FixedEffects),
   RandomEffects = cnRandomEffects, 
   Models=Models, sigmasq.start = 1,  numChains = 5,
   lengthChains=2500,burnin.start = 1,
   DIC.Only=FALSE,  tauPriorFile = NULL, DoBayesSpike = FALSE, 
   Verbose = 0);

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb 
                               ## corresponding to our true model

plot(ADO$cent.chains);  ## Plot the centered chains to understand
summary(ADO$cent.chains);  ## Get Summary information on a mean centered parameters
plot(ADO$raw.chains);  ## Plot the chains to understand
summary(ADO$raw.chains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$raw.chains); ## Using Library_diallel_helper, plot chains.

plot.corr(crosscorr(ADO$raw.chains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
