\name{DiallelOb}
\alias{Diallel Object}
\alias{DiallelOb}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Diallel Object}
\description{
  Member of Full Diallel Object, in which gibbs samples and a "diallel model" is stored.
  
  Within Full Diallel Analyze clas, the list "MyFullDiallelAnalyze$AllDiallelObs"
   contains multiple diallel objects.
   
  Within Full Diallel Simulator, the single object "MyFullDiallelSimulator$ADiallelOb"
   contains a single model
}
\usage{
  DiallelOb(AFD, numj = defaultnumj,ajModel = 1, MotherModel = 0, 
         CrossModel = 1, SexModel = 0, BetaInbredLevel = 0, 
         BetaInbredTimesSex = 0, dfTNoise = NULL, ...);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{AFD}{Full Diallel Analyze/Simulate Object that will hold this DiallelOb}
  \item{ajModel}{Flag determining inclusion of additive coefficients, 
  
  0 if no additive, 1 if additive}
  \item{MotherModel}{Flag determining inclusion of maternal coefficients, 
  
  0 if no mother specific, 1 if mother specific}
  \item{CrossModel}{Flag determining inclusion of cross (off-diagonal) coefficients v,w
  
    Level 0, no hybrid effects,\cr
    Level 1 is equivalent to inbreeding BetaInbredLevel = 1 model,\cr
    Level 2 is turning on v = symmetric hybrid effects, \cr Level 5 turns on w, asymmetric hybrid effects,\cr
      and Level 8 concerns all off-diagonal hybrid effects as levels.}
  \item{SexModel}{Flag determining which model parts get sex specific effects, 0 no sex effects,
  
    1 sex interaction with additive \cr  
    2 sex interaction with maternal effects   \cr
    3 sex interaction with Inbred-strain specific (same as BetaInbredTimesSex = 3) \cr
    4 sex interaction with symmetric-strain hybridization \cr
    5 sex interaction with asymmetric-strain hybridization \cr
    6 sex interaction with all-strain off-diagonal hybriidzation}
  \item{BetaInbredLevel}{Flag determing a single fixed effect for inbreeding, or else
    a vector of strain specific inbreeding effects.  
    
    0 is no Inbreeding effect, \cr
    1 is a universal all strain inbreeding effect "B", \cr
    3 supports a strain-specific inbreeding effects "b_j" }, 
  \item{BetaInbredTimesSex}{Flag determing a gender specific single fixed effect for inbreeding, or else
    a vector of strain/gender specific inbreeding effects 
    
    0 is no Inbreeding effect interaction with sex, \cr
    1 is a universal all strain inbreeding sex interaction effect "B_s"   \cr
    3 supports a strain-specific inbreeding sex interaction effects "bs_j"}
  \item{dfTNoise}{Degrees of Freedom for t-distribution noise if desired}
  \item{numj}{Total Number of Strains under consideration}
}
\details{
  DiallelOb has several default functions including "copy" and "finalize" for 
  deleting.  
  
  Hidden members .numj (number of diallel strains), .ajModel, .MotherModel,
    .CrossModel develop from model cross levels
    .SexModel, .BetaInbredLevel, .BetaHybirdLevelTimesSex
    
  DiallelObs are created using "\code{\link{SetupAnalysisMethods}}"  on a 
    \code{\link{FullDiallelAnalyze}} Object
  
   
   # ajLevel:     \cr                  
##  0 if no Aj terms, 1 if        \cr
## MotherModel: MotherModel = 0 if no mother effect to study, \cr
##     0 if no Mother Effect    \cr
##     1 if j dependent           \cr
##     2 if so complicated as to make Thetaij discrete from Thetaji;\cr

\cr
## Cross Epistatsis Models          \cr
##   0 For No Cross Terms             \cr
##   1 For dominance Hybridj model\cr
##   2 For all pooled\cr
##   3 for pool Theta_ij sim N(0, taui^2i * tauj^2)\cr
##   4 for pool Theta_ij sim N(0, taui^2 + tauj^2);\cr
##   5 for All Pooled with Counter Cross terms\cr
##   6 for pool Theta_ij sim N(0, taui^2i * tauj^2), counter cross\cr
##   7 for pool Theta_ij sim N(0, taui^2i + tauj^2), counter cross\cr
##   8 for All Pooled, ij and ji all separate\cr
##   9 for separate ij and ji both N(0, taui^2i * tauj^2)\cr
##   10 for separate ij and ji both N(0, taui^2i + tauj^2)\cr

\cr
## Beta Hybrid Level\cr
## 0 No Beta Hybrid\cr
## 1 BetaHybrid ~ N(0, tauD);\cr
## 2 BetaHybridj ~ N(0, tauDj);  tauDj ~ InvGamma\cr
## 3 BetaHybridj ~ N(BetaHybrid, tauDj), tauDj ~InvGamma, BetaHybrid ~ tauD; \cr

\cr
## Sex Models\cr
##  0 for No Sex\cr
##  1 for Additive Only Sex\cr
##  2 for Cross Sex aj\cr
##  3 for Cross Sex aj Cross Sex mj\cr
##  4 for Cross Sex aj Cross Sex dj\cr
##  5 for Cross Sex aj Cross Sex mj, Cross taui tauj\cr

\cr
## BetaHybrid * Sex \cr
##  0 No BetaHybrid * Sex\cr
##  1 Else \cr
##BetaInbredTimesSex = 1;\cr

}
\value{
    Object  returns an DiallelOb
    
    This Object contains default members:
    \item{ListFunctions}{List of Functions available to this DiallelOb}
    \item{ListElements}{List of Elements available to this DiallelOb}
    \item{cent.chains}{Output Chains, with shrink towards group mean, for official analysis.  Runs
       "AutoMFFunction" on CodaChains, to create this if not created yet} 
    \item{raw.chains}{Original Output chains from MCMC sampler.  Setup by SetupChains}
    \item{DpBeta}{A matrix that enables mapping for Groups to their Tau parameters}
    \item{XtX}{Square matrix of model member covariates}
    \item{dfTNoise}{Number of degrees of freedom of model for t-noise distribution}
    \item{VecOWeights}{Weights at current iteration if dfTNoise > 0}
    \item{Model}{Alias for getModel() function, returns in text model of current DiallelOb}
    \item{.numj}{Number of Diallel Strains}
    \item{.AFD}{Hidden Link to Full DiallelAnalyze object that generated this DiallelOb}
    \item{.iOb}{Number of Diallel Object in .AFD$AllDiallelObs if applicable}
    \item{PosteriorPredictionsMeanSummary}{Summary Table of predictions for mean of squares}
    \item{PosteriorPredictionsErrorSummary}{Summary Tables of predictions for mean of squares + error draws}
    \item{PosteriorPredictionsMeanChains}{Coda Chains of predictions for mean of squares}
    \item{PosteriorPredictionsErrorChains}{Coda Chains of predictions for mean of squares + error draws}   
    \item{PostKeeper}{Coda Chains kept by Posterior Analyses} 
    \item{ListUnknownGender}{List of which specimens have unknown sex}
    \item{ImputeGenderChains}{Coda Chains of imputed sex for specimens unknown}
    \item{GXX}{Switching vector useful in imputing unknown gender}
    \item{ImputedGender}{Current Imputed Gender state}
    \item{PosteriorHSqChains}{Coda Chains of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, 
      uses function \code{\link{PosteriorHSq}} }
    \item{PosteriorHSqAll}{Whole derived components of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, 
      uses function \code{\link{PosteriorHSq}} }
    \item{PosteriorHSqPoint}{HSq based upon point estimates of tau^2 of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, 
      uses function \code{\link{PosteriorHSq}} }
    \item{PosteriorHSqChainsImproper}{Coda Chains of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, using flat weights for tau^2,
      uses function \code{\link{PosteriorHSq}} }
    \item{PosteriorHSqChainsWithFixed}{Coda Chains of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, 
      includes heritability component for the fixed effects as well as random effects,
      uses function \code{\link{PosteriorHSq}} }    
    \item{PosteriorHSqTable}{Summary Table of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, 
      uses function \code{\link{PosteriorHSq}} }
    \item{PosteriorHSqTableImproper}{Summary Table of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, using flat weights for tau^2,
      uses function \code{\link{PosteriorHSq}} }
    \item{PosteriorHSqTableWithFixed}{Summary Table of posterior draws for HSq, 
      or heritability value for each component for future independent diallels, 
      includes heritability component for the fixed effects as well as random effects,
      uses function \code{\link{PosteriorHSq}} } 
  \item{PosteriorRSqChains}{Coda Chains of posterior draws for RSq, 
      or variance explained in the current diallel value for each component,
      uses function \code{\link{PosteriorRSq}} }
    \item{PosteriorRSqChainsImproper}{Coda Chains of posterior draws for RSq, 
       or variance explained in the current diallel value for each component, using flat weights for XtX,
      uses function \code{\link{PosteriorRSq}} }
    \item{PosteriorRSqChainsWithFixed}{Coda Chains of posterior draws for RSq, 
       or variance explained in the current diallel value for each component,
      includes variance component explained bythe fixed effects as well as random effects,
      uses function \code{\link{PosteriorRSq}} }    
    \item{PosteriorRSqTable}{Summary Table of posterior draws for RSq, 
      or variance explained in the current diallel value for each component,
      uses function \code{\link{PosteriorRSq}} }
    \item{PosteriorRSqTableImproper}{Summary Table of posterior draws for  RSq, 
       or variance explained in the current diallel value for each component, using flat weights for XtX,
      uses function \code{\link{PosteriorRSq}} }
    \item{PosteriorHSqTableWithFixed}{Summary Table of posterior draws for RSq, 
       or variance explained in the current diallel value for each component,
      includes variance component explained bythe fixed effects as well as random effects,
      uses function \code{\link{PosteriorRSq}} }    

    
    This Object contains Functions:
     \item{CreateGotList}{Sets up Diallel Ob towards Got List type Model, a key setup function that uses}
     \item{ReplaceAndRerunUpdatedChains}{Reruns chains on DialleObject}
     \item{SetupChains}{Sets up Gibbs Sampler Chains for DiallelObject}
     \item{RecalculateLikelihood}{Recalculate Likelihood values at places in the chains}
     \item{SetupHybridNoise}{Not sure what this does }
     \item{RunChainsCC}{finally runs the chains analysis in C++ code }
     \item{CreateNetBetas}{Not sure what NetBetas are?}
     \item{PrintItersSamples}{Old code that prints results of iterations of sampler for user information}
     \item{CalcDIC}{Calculate DIC value for positons in Code}
     \item{RunEM}{Runs EM maximization for DiallelOb if it is an EM matrix}
     \item{CalcDICC}{Calculate DIC using C for the DiallelObject}
     \item{SetupUnknownGender}{Setup DiallelObject for dealing with missing Genders of some samples}
     \item{GetModel}{Returns a vector with flags recording model of the DiallelOb}
     \item{finalize}{Delete the DiallelOb for garbage collection}
     \item{Copy}{Return a copy of this Diallel Object}
     \item{summary.table}{Attempts to summarize the Diallel Object with valuable information}    
     \item{PosteriorRSq}{Derive R-squared scores for fitted chains describing amount of variance explained}
     \item{AutoMFFunction}{Create Shrunk Version of the current Coda Chains}
     \item{PosteriorHSq}{Derive approximate 
       heritability assigned to traits according to fitted tau^2_q parameters}
     \item{\code{\link{TwoDiallelPlot}}}{Plots Diallel Predicted checkerboards.  Three options
       \code{FemalePlot = TRUE, MaleAgainstFemale = FALSE, PlotOnlyObserved = FALSE} are set to
       do different two by two checkerboard comparisons}
    \item{\code{\link{PosteriorHSq}}}{Calculate H^2 for diallel, or estimated future heritability in future independent Diallel experiements}
    \item{\code{\link{PosteriorRSq}}}{Calculate R^2 for diallel, or proportion of observed data explained by fitted components,
      note that there is overlap so RSq does not sum to 1}
     \item{\code{\link{PosteriorPredSummary}}}{Create a summary of posterior predictions for diallel Squares}
     \item{\code{\link{PlotStrawPlot}}}{Automatically plot Straw plot for this model.  Straw plot is a plot of strain effect means color coded between strains}
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{FullDiallel}}, \code{\link{DiallelAnalyzer}} }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}

## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)

#######################################################3
## Setup Analysis
##
## Allocate AFD : FullDiallelAnalyze Object
AFD  <- FullDiallelAnalyze(Y = Y, 
      SexVector = SexInputVec,
      Listjk = ListjkInput, dfStart = .5, mStart = .5,
      InFixedEffects = AFDS$FixedEffects,
      InRandomEffects = ARandomEffects);

## Assemble a List of models to be considered by AFD
##  This is a k * 6 matrix;
##
## In our case, let k = 1, and supply AFD the "true" simulation model 
ExampleModelsList = matrix(c(ajModel, MotherModel, CrossModel,
      SexModel, BetaInbredLevel, BetaInbredTimesSex), 1, 6);
      Sigma = 1;  
      names(Sigma) = "SigmaAll";

## Setup Analysis Methods for AFD, including allocating for MCMC chains
## "Sigma" is starting Sigma anticipated noise value
SetupAnalysisMethods(AFD, ExampleModelsList, Sigma = Sigma,
      numChains = 3, lengthChains = 2000);

#######################################################
## This will run the MCMC sampler on Model of choice
t1 = proc.time();
  RunChains(AFD);
t2 = proc.time();                  
print(t2-t1); ## Print Run time

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb corresponding to our model

plot(ADO$cent.chains);  ## Plot the centered version of DiallelOb's chains;
plot(ADO$raw.chains);  ## Plot the raw version of DiallelOb's chains;

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")


#################################################################################
##  HSq and RSq
##
##   HSq is our future heritability metric, based upon estimated tau^2 parameters,
##    it assumes independence of  future diallels.
##
##   RSq relates the amount of variation in Y explained by a covariate.  Note, that
##    because of correlation between terms, sum of RSq terms can be more or less than
##    total variation of Y.  
##
## Give a table summarizing beliefs in heritability ratios for future independent diallels.
##  expresses residual variance in terms of 
AFD$AllDiallelObs[[1]]$PosteriorHSqTable

## Inner method that calculates that HSq
OtherPosteriorChains = AFD$AllDiallelObs[[1]]$PosteriorHSq(UseFixed=TRUE, ProperRatio=TRUE,
      AFD = AFD, startiter = AFD$burin, thin = AFD$thin)
      
## Now look at RSq as a different metric
summary(AFD$AllDiallelObs[[1]]$PosteriorRSqChains);
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
