\name{FullDiallelEMAnalyze}
\alias{FullDiallelEMAnalyze}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Create FullDiallelAnalyze EM Object }
\description{
  Create "FullDiallel" specifically of Analyze type
}
\usage{
FullDiallel(numj=defaultnumj,  Y = NULL, InListjk = NULL, InSexVector=NULL,
  InFixedEffects = NULL, nameFixedEffects = NULL,
  InRandomEffects = NULL, nameRandomEffects = NULL,
  InRandomPrintEffects=NULL, dfTNoise = NULL,
  Indf = NULL, Inm = NULL, IntauFixed=NULL, 
  Indftau = NULL,Inmtau = NULL,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{Y}{Dependendent Vector}
  \item{numj}{ Number of total Strains in Diallel}
  \item{Indf}{ A "RConstantsList", object representing prior inverse gamma degrees of freedom for parameters. 
    default NULL sets all values at .5}
  \item{Inm}{A "RConstantsList",object representing prior inverse gamma scaling for parameters. 
    default NULL sets all values at .5 }
  \item{dfTNoise}{If dfTNoise >0, then a outlier procedure based upon t-distributed noise is used}
  \item{InSexVector}{A vector declaring 0 if mouse is female, 1 if male}
  \item{InListjk}{A two column list, first column, strain of mother, second column, strain of father}
  \item{IntauFixed}{In Starting parameters for the fixed coefficients}
  \item{verbose}{ Decision to print actions to console }
  \item{Indftau}{Hierarchical prior information}
  \item{Inmtau}{Hierarchical prior information}
}
\details{
  All Analyses will use "FullDiallelAnalyzeEM", an inherited class of Full Diallel
}
\value{
    Object  
     \item{FullDiallel Object}{An S3 Class designed to test multiple diallel models on the same dataset}
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "FullDiallelEMAnalyze"  is a class that will analyze a dataset
}
\seealso{ FullDiallelAnalyze, FullDiallelSimulate, DiallelOb }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);


###############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info);
   
## Get a summary of the Coda Chains for first Diallel Model
summary(AFD$AllDiallelObs[[1]]$CodaChains)
plot(AFD$AllDiallelObs[[1]]$CodaChains);
plot.hpd(AFD$AllDiallelObs[[1]]$CodaChains);

## Here "AFD" is the FullDiallel Analyze object
##   AFD$AllDiallelObs is a list of DiallelObs
##   AFD$AllDiallelObs[[1]] is the first Diallel Obs

##########################################################
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}

## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)

#######################################################3
## Setup Analysis
##
## Allocate AFD : FullDiallelAnalyze Object
AFD  <- FullDiallelEMAnalyze(Y = AFDS$Y, 
      SexVector = SexInputVec,
      Listjk = ListjkInput, dfStart = .5, mStart = .5,
      InFixedEffects = AFDS$FixedEffects,
      InRandomEffects = ARandomEffects);

## Assemble a List of models to be considered by AFD
##  This is a k * 6 matrix;
##
## In our case, let k = 1, and supply AFD the "true" simulation model 
ExampleModelsList = matrix(c(ajModel, MotherModel, CrossModel,
      SexModel, BetaInbredLevel, BetaInbredTimesSex), 1, 6);
      Sigma = 1;  
      names(Sigma) = "SigmaAll";


##  Options for Chains:
##   Set "Wanted" in Analysis methods so that only certain variables in model are taken
##   Set "Extractor" in Analysis methods so that functions of variables in model are taken.
##   Use "AFD$On$State" to learn about variables in the State Vector for
##    Current DiallelModel
##   (Note, that is "AFD$AllDiallelObs[[AFD$OnObs]]$getState()")
OptWanted = NULL; OptExtractor = NULL;
##OptWanted = c("Mu", "aj:3", "Sigma:1")
## Setup Analysis Methods for AFD, including allocating for MCMC chains
## "Sigma" is starting Sigma anticipated noise value'
if (FALSE) {
 OptExtractor = function(State) {
   returnVec = c(State[["Mu"]], State[["Sigma:1"]],
     mean(State[substr(names(State),1,nchar("aj")) == "aj"])
   );
   names(returnVec) = c("Mu", "Sigma:1", "Meanaj");
   return(returnVec);
 }
}
SetupAnalysisMethods(AFD, ExampleModelsList, Sigma = Sigma,
      numChains = 3, lengthChains = 2000,Wanted = OptWanted,
      Extractor = OptExtractor);

#######################################################
## This will run the MCMC sampler on Model of choice
t1 = proc.time();
  RunEMcc(AFD$AllDiallelObs[[1]]);
t2 = proc.time();                  
print(t2-t1); ## Print Run time

TrueBeta = AFD$AllDiallelObs[[1]]$Beta * 0;
MT = match(names(AFDS$ADiallelOb$Beta), names(TrueBeta));
TrueBeta[MT[!is.na(MT)]] = AFDS$ADiallelOb$Beta[!is.na(MT)];
Fit = AFD$AllDiallelObs[[1]]$Beta;
Fit - TrueBeta;

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb corresponding to our model

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot.corr(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
