\name{FullDiallelSimulate}
\alias{FullDiallelSimulate}
\alias{SimulateCompetingDiallels}
\alias{Simulations}
\alias{Simulate}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Create FullDiallel Object of type FullDiallelSimulate }
\description{
  Create "FullDiallelSimulate" object intended for simulation.
}
\usage{
FullDiallelSimulate(numj=defaultnumj,  InListjk = NULL, InSexVector=NULL,
  InFixedEffects = NULL, nameFixedEffects = NULL,
  InRandomEffects = NULL, nameRandomEffects = NULL,
  InRandomPrintEffects=NULL, dfTNoise = NULL,
  Indf = NULL, Inm = NULL, IntauFixed=NULL, 
  Indftau = NULL,Inmtau = NULL,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{numj}{ Number of total Strains in Diallel}
  \item{Indf}{ A "RConstantsList", object representing prior inverse gamma degrees of freedom for parameters. 
    default NULL sets all values at .5}
  \item{Inm}{A "RConstantsList",object representing prior inverse gamma scaling for parameters. 
    default NULL sets all values at .5 }
  \item{dfTNoise}{If dfTNoise >0, then a outlier procedure based upon t-distributed noise is used}
  \item{InSexVector}{A vector declaring 0 if mouse is female, 1 if male}
  \item{InListjk}{A two column list, first column, strain of mother, second column, strain of father}
  \item{IntauFixed}{In Starting parameters for the fixed coefficients}
  \item{verbose}{ Decision to print actions to console }
  \item{Indftau}{Hierarchical prior information}
  \item{Inmtau}{Hierarchical prior information}
}
\details{
  Simulations of Diallels generated using "FullDiallelSimulate", an inherited class of Full Diallel
  
  Large scale simulations and summary analysis is possible with this package, though materials are left
    shortly documented.  File "SimulateCompetingDiallels.R" gives a directory structure based approach 
    to simulating Diallels of many different formats, using file "RParrallelPack.R" to justify use
    of multiple simultaneous scripts in multi-core simulation studies.  "SourcePrintingFunctions" and
    "ProcessDiallelOutput.R" give ways to generate Latex tables out of the analysis.  Code for running
    the analyses used in original BayesDiallel paper is locked in a if (FALSE) statement in 
    examples section of this man document.  Warning is that it has only been run on our system.
    also requires "BayesSpike package" for BayesSpike related simulations, as well as pacakge
    "hglm" for "hglm" related simulations.
}
\value{
    Object  
     \item{FullDiallel Object}{An R.oo/S3 Class designed to test multiple diallel models on the same dataset}
     \item{m}{This is an important RandomEffects object holding "m" (mean) values for inverse gamma distributions in all model priors (sim or not)}
     \item{df}{This is an important RandomEffects object holding "df" (degree of freedom) values for inverse gamma distributions in all model priors (sim or not)}
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "FullDiallelSimulate"  is a class that will analyze a dataset
}
\seealso{ \code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallel}}, 
  \code{\link{DiallelOb}}, \link{BayesDiallel} }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}

## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)

#######################################################3
## Setup Analysis
##
## Allocate AFD : FullDiallelAnalyze Object
AFD  <- FullDiallelAnalyze(Y = Y, 
      SexVector = SexInputVec,
      Listjk = ListjkInput, dfStart = .5, mStart = .5,
      InFixedEffects = AFDS$FixedEffects,
      InRandomEffects = ARandomEffects);

## Assemble a List of models to be considered by AFD
##  This is a k * 6 matrix;
##
## In our case, let k = 1, and supply AFD the "true" simulation model 
ExampleModelsList = matrix(c(ajModel, MotherModel, CrossModel,
      SexModel, BetaInbredLevel, BetaInbredTimesSex), 1, 6);
      Sigma = 1;  
      names(Sigma) = "SigmaAll";

## Setup Analysis Methods for AFD, including allocating for MCMC chains
## "Sigma" is starting Sigma anticipated noise value
SetupAnalysisMethods(AFD, ExampleModelsList, Sigma = Sigma,
      numChains = 3, lengthChains = 2000);

#######################################################
## This will run the MCMC sampler on Model of choice
t1 = proc.time();
  RunChains(AFD);
t2 = proc.time();                  
print(t2-t1); ## Print Run time

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb corresponding to our model

plot(ADO$cent.chains);  ## Plot the chains to understand
summary(ADO$cent.chains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$cent.chains); ## Using Library_diallel_helper, plot chains.


plot(crosscorr(ADO$cent.chains)) ## Look at cross correlations of mean centeredcoda chains
plot(crosscorr(ADO$raw.chains)) ## Look at cross correlations of raw coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")

#########################################################################################
## Large Scale Simulations
##  It can take days for this code to run, but it should be possible to run the code
##   on multiple cores, starting and stopping with the finish of each analysis object
##  One must change Directory structure: aka, set a directory "SAVEHOME" in which
##   all of the analysis of simulation files are saved and updated.
##  BayesDiallel::SetSaveHomes() is not expected to pick a good directory on arbitrary
##    systems and should be rewritten pending a users needs and computing equipment.
##
## Note that this code, though available in BayesDiallel package is sparsely documented
##   in man files.  Source file "SimulatCompetingDiallels.R" is where most of 
##   the script code for designing multiple Diallel measures exists
##   File RParrallelPack.R is a source file dealing with parrallizable summaries of 
##   Gibbs sampler experiments, and defines "OutCodaAnalyses" and "OutLMAnalyses"
##   table objects that can construct directory structures for this simulation
##   "ProcessDiallelOuput.R" then contains most source code for generating tables
##   (as in "WriteWillTable()" and "WriteParamFightTable()")
##
##
#################################################################################
##  Simple Model
##
##   only aj is active
if (FALSE) {
MySimUName = "NewAsOldGriffMX";
ModelsIn <- c("Griff", "GriffSat", "DiallelLarge",
  "DiallelDICMin", "DiallelTRUE", "DiallelSpike",
  "DiallelLargeOutlier3", "DiallelLargeOutlier6",
  "DiallelLargeOutlier15", "DiallelSpikeOutlier6", "Diallelhglm")
if (FALSE) {
ModelsIn <- c("Griff", "DiallelLarge", "DiallelTRUE",
  "Diallelhglm")
}  
DOSIM = TRUE;  certainNoPrint = 1;
numj = 8;
ajVector = c(-10,-8,-4,-1,1,3,7,12);
motherjVector = rep(0,8)
GendermotherjVector = rep(0,8)
dominancejVector = rep(0,8)
SSigmaSq = 120;
GenderOn = FALSE;
Mu = 7; GenderMu = 0; BetaHybrid =0; BetaHybridTimesGender = 0;


SetBetaMethod = 1;
ajModel = 1; MotherModel = 1;  CrossModel = 2;
tauaj = 5; tauGenderaj = 4;
taumotherj = 3; tauGendermotherj = 4; taudominancej=0;
tauGenderdominancej = 0; tauSymCrossjk = 0; tauGenderSymCrossjk = 0;
tauASymCrossjkDkj = 0; tauGenderASymCrossjkDkj = 0;  tauAllCrossjk = 0;
tauGenderAllCrossjk = 0;

PiAPrior = c(10,10,20,20);

InBalance = c(0,0);
nReps = 5; NumRepetitions = nReps;




try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
require(BayesSpike)

TARGET = 400;
PRINTEVERY = 10 + abs(round(30 + 8*rnorm(1,0,1)));
PRINTEVERY = 2;

UpdateVerbose = FALSE;
##UpdateVerbose = TRUE;


BSLengthChains = 2000;
BSNumChains = 5;
NumSimPred = 1000;
start = 250;

##if (!exists("args") || is.null(args) || is.na(args) ) {
args <- commandArgs(TRUE)
##} #args = c(1,666,1,3,4,1,3)


if (length(args) > 0) {
 argList1 = args[1];
} else {
 argList1 = 0;
}

argList = as.numeric(args[2:length(args)]);

BayesDiallel:::SetSaveHomes(); ### Creates a Save Homes directory important to computer path
## Set your own SAVEHOME in case your directory does not follow the default!

AllDir <- paste(SAVEHOME, "//September23NewAsOldGriffMXnLargeNoise", tSeq(nReps), sep="");
dir.create(AllDir, showWarnings=FALSE);

BSSaveDir <- paste(BSSaveDir, "//September23NewAsOldGriffMXnLargeNoise", tSeq(nReps), sep="");
dir.create(BSSaveDir, showWarnings=FALSE);
MyOutDir = paste(DropHome, "//DiallelWork//NAOGMXLargeNoiseSim//", sep="");
dir.create(MyOutDir, showWarnings=FALSE);
MyOutDir = paste(DropHome, "//DiallelWork//NAOGMXLargeNoiseSim//September23//", sep="");
dir.create(MyOutDir, showWarnings=FALSE);

BayesDiallel:::SetIdentifierList();
BayesDiallel:::SimulateCompetingDiallels(Verbose = 4);

BayesDiallel:::ReadASmallSimmer(MySimmer, MaxReadLength = 400, KillLockFiles = FALSE);

Verbose = 0;
RunASimulation(JustStart = TRUE);

print("Now Running One Simulation");
RunASimulation();

for (ii in 1:40) {
RunASimulation();
}

if (FALSE) {
MakeOAs();
TrueParamVec = TrueParameterVec;
  ReadIter = length(MySimmer$SimL[[1]]$MyOA$MeanVals$ReadTable[,1])
  if (!exists("HitSimPreds") || is.null(HitSimPreds)) {
    SimHitSimPreds();
  }
  ReadIter = 1; SigLevel = .2; FileName = paste(MyOutDir, "Table1.tex", sep="")
  KeepRDs = -1; KeepGAs = -1;
 
 WriteWillTable(TrueParamVec, GriffingOAs=GriffingOAs,
  BayesDiallelOAs=BayesDiallelOAs, FileName =paste(MyOutDir, "Table1.tex", sep=""), SigLevel = .2,
  ReadIter = NROW(MySimmer$SimL[[1]]$MyOA$MeanVals$ReadTable), TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds=HitSimPreds)
 WriteWillTable(TrueParamVec, GriffingOAs=GriffingOAs,
  BayesDiallelOAs=BayesDiallelOAs, FileName =paste(MyOutDir, "Table1a.tex", sep=""), SigLevel = .2,
  ReadIter = NROW(MySimmer$SimL[[1]]$MyOA$MeanVals$ReadTable), TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds=HitSimPreds,
    KeepGAs = c(1,2), KeepRDs =c(1,2,3) )
 WriteWillTable(TrueParamVec, GriffingOAs=GriffingOAs,
  BayesDiallelOAs=BayesDiallelOAs, FileName =paste(MyOutDir, "Table1b.tex", sep=""), SigLevel = .2,
  ReadIter = NROW(MySimmer$SimL[[1]]$MyOA$MeanVals$ReadTable), TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds=HitSimPreds,
    KeepGAs = NULL, KeepRDs = c(4,5,6,7,8) )  
}

RunSimulationsUntilTarget(Verbose =2, SepRuns = 20)

MakeOAs();
if (length(list.files(DropHome)) > 0) {
  HDir = paste(DropHome, "/DiallelWork/",sep="");
} else {
  HDir = paste(DropHome, "/DiallelWork/",sep="");
}
dir.create(paste(HDir, "September29NewAsOldGriffMXnLargeNoise5/", sep=""), showWarnings=FALSE);

FileName= paste(MyOutDir, "Table1.tex", sep="")
MedianSigLevel = .02;  MIPSigLevel = .2;  SubSetIters = NULL;  TargetRDTable = TargetRDTable;  TargetRDProb = TargetRDProb;  HitSimPreds = NULL;
GiveTargetGroups = 0;  roundTo = 3;  PredDiff = TRUE; roundShort = 1;  GiveTargetGroups = c("aj", "motherj", "dominancej", "SymCrossjk")

WPT <- WriteParamFightTable(GriffingOAs, BayesDiallelOAs, FileName= paste(MyOutDir, "/WholeTable1.tex", sep=""),
  MedianSigLevel = .02, MIPSigLevel=.1,
  SubSetIters = NULL, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds = NULL,
  GiveTargetGroups = c("aj", "motherj", "dominancej", "SymCrossjk")
, roundTo = 3, PredDiff = TRUE)

WPT <- WriteParamFightTable(GriffingOAs, BayesDiallelOAs, FileName= paste(MyOutDir, "WholeTable1a.tex", sep=""),
  MedianSigLevel = .02, MIPSigLevel=.1,
  SubSetIters = NULL, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds = NULL,
  GiveTargetGroups = c("aj", "motherj", "dominancej", "SymCrossjk")
, roundTo = 3, PredDiff = TRUE, KeepRDs= c(1,2,3,4), KeepGAs = c(1,2))

WPT <- WriteParamFightTable(GriffingOAs, BayesDiallelOAs, FileName= paste(MyOutDir,"WholeTable1b.tex", sep=""),
  MedianSigLevel = .02, MIPSigLevel=.1,
  SubSetIters = NULL, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds = NULL,
  GiveTargetGroups = c("aj", "motherj", "dominancej", "SymCrossjk")
, roundTo = 3, PredDiff = TRUE, KeepRDs = c(5,6,7,8), KeepGAs = NULL)

ATS <- RunHGLMForSigma(200);
}
##############################################################################################
##  Simulation Model
##   Simulation Code for a larger model
if (FALSE) {
MySimUName = "WillSimB";
ModelsIn <- c("Griff", "GriffSat", "DiallelLarge",
  "DiallelTRUE", "DiallelSpike",
  "DiallelLargeOutlier3", "DiallelLargeOutlier6",
  "DiallelLargeOutlier15", "DiallelSpikeOutlier6", "Diallelhglm")
  ##"DiallelDICMin", 
DOSIM = TRUE;  certainNoPrint = 1;
numj = 8;
ajVector = sqrt(.52) * c(-10,-8,-4,-1,1,3,7,12);
motherjVector = 1.5*c(0.3325, 0.5425, -0.8475, 2.4125, 3.2525, -4.7275, 0.0625, -1.0275);
GendermotherjVector = 6*c(0.75, 0.65, 0.85, -2.35, 0.15, -0.05, -0.95, 0.95);
dominancejVector = 19*c(0.4125, -0.3875, -0.3875, 0.2125, 0.3125, -0.2875, 0.0125, 0.1125);
GenderASymCrossjkDkjVector = 0 * c(0.848, 0.258, 0.288, -0.692, 0.978, -0.462, -0.712, -0.052, 1.328, -0.572,
  -0.172, -0.272, 0.698, -0.122, 0.308, 2.048, -0.372, -0.092, -0.982, 0.158, -1.762, -0.112,
  0.288, -0.492, -1.022, -1.172, 0.568, 1.288)
SSigmaSq = 120;
SSigmaSq = 120;

tauSq = c(2*sd(ajVector)^2, sd(motherjVector)^2, sd(GendermotherjVector)^2, 
  2*sd(dominancejVector)^2, sd(GenderASymCrossjkDkjVector)^2, SSigmaSq);
  tauSq / sum(tauSq);
GenderOn = TRUE;
Mu = 7; GenderMu = 4; BetaHybrid =-4; BetaHybridTimesGender = 0;


KillVerboseChains = TRUE;
SetBetaMethod = 1;
ajModel = 1; MotherModel = 1;  CrossModel = 2;
tauaj = 5; tauGenderaj = 4;
taumotherj = 3; tauGendermotherj = 4; taudominancej=0;
tauGenderdominancej = 0; tauSymCrossjk = 0; tauGenderSymCrossjk = 0;
tauASymCrossjkDkj = 0; tauGenderASymCrossjkDkj = 0;  tauAllCrossjk = 0;
tauGenderAllCrossjk = 0;

PiAPrior = c(10,10,20,20);

InBalance = c(0,0);
nReps = 5;


PriorProbAllTau = .5;
PriorProbAllFixed = .5;





try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
require(BayesSpike)



TARGET = 1000;
PRINTEVERY = 10 + abs(round(30 + 8*rnorm(1,0,1)));
PRINTEVERY = 2;

UpdateVerbose = FALSE;
##UpdateVerbose = TRUE;


BSLengthChains = 2000;
BSNumChains = 5;
NumSimPred = 1000;
start = 400;

##if (!exists("args") || is.null(args) || is.na(args) ) {
args <- commandArgs(TRUE)
##} #args = c(1,666,1,3,4,1,3)


if (length(args) > 0) {
 argList1 = args[1];
} else {
 argList1 = 0;
}

argList = as.numeric(args[2:length(args)]);

BayesDiallel:::SetSaveHomes(); ### Creates a Save Homes directory important to computer path
##SAVEHOME = (DropHome)


##    MeanDiff=MeanDiff, MedianDiff=MedianDiff,
##    WidthCI=WidthCI, CovCI=CovCI, LowCI=LowCI, UpCI = UpCI,
##    MeanVals = MeanVals, MedianVals=MedianVals, 
##    SmPVal=SmPVal, MIPVal=MIPVal, SDVal = SDVal,
##    PredMeans = PredMeans, PredMedians=PredMedians, PredMeanDiffs=PredMeanDiffs,
##    PredMedianDiffs = PredMedianDiffs,
##    PredSimsMeanTable=PredSimsMeanTable, PredSimsMedianTable=PredSimsMedianTable,
##    PredSimsMeanTableWrittenLength=0, PredSimsMedianTableWrittenLength=0,
##    PreferMeanMedian = PreferMeanMedian,

OnlyNeedTablesList = c("MeanVals", "MedianVals", "TrueParamTable",
  "MIPVal", "SmPVal", "SDVal", "PredMeanDiffs", "PredMedianDiffs",
  "PredSimsMeanTable", "PredSimsMedianTable");
  OnlyNeedTablesList = NULL;
  ##SAVEHOME = "/Users/lenarcic/Documents/OtherComputerSims/SavageDiallel";
##
##dir.create(AllDir, showWarnings=TRUE);
## SAVEHOME = "zep2.med.unc.edu
 AllDir <- paste(SAVEHOME, "//WillSimBSept27", sep="");
 dir.create(AllDir, showWarnings=TRUE);
 BSSaveDir <- paste(BSSaveDir, "//WillSimBSept27", sep="");
 DirSave = paste(DropHome, "//DiallelWork//WillSimBSept27", sep="");
 dir.create(DirSave, showWarnings=FALSE);
 
##
##dir.create(BSSaveDir, showWarnings=TRUE);


BayesDiallel:::SetIdentifierList();

KillLockFiles = TRUE; MaxReadLength = 200;
BayesDiallel:::SimulateCompetingDiallels(Verbose = 4, MaxReadLength = 200);
BayesDiallel:::ReadASmallSimmer(MySimmer, MaxReadLength = MaxReadLength, KillLockFiles = FALSE);

print("Now Running One Simulation");
RunASimulation();
      
RunSimulationsUntilTarget(Verbose =2, SepRuns = 20)

MakeOAs();
FileName=paste(DirSave, "//Table1.tex", sep="");
  MedianSigLevel = .01; MIPSigLevel=.2;
  SubSetIters = NULL;  HitSimPreds = NULL;
  GiveTargetGroups = 0; roundTo = 3; PredDiff = TRUE;
  KeepRDs = -1; KeepGAs = -1;
  
#this = GriffingOAs[[jj]];
L2OrL1 = "L2";
        MeanOrMedian="MEAN"; ParamGroup = GiveTargetGroups[ii]; 
        ParamVal = NULL; AllFixed = NULL;
        ITERS = NULL; NonNAIters = FALSE;
        Denominator = c("None","T1", "T2")[1]; ShiftDiff = TRUE;
        quantiles=c(.025,.975)
        ReadGIters = list();
        for (ii in 1:length(GriffingOAs)) { ReadGIters[[ii]] = (1:NROW(GriffingOAs[[1]]$MeanVals$ReadTable))}
         ReadRDIters = list();
        for (ii in 1:length(BayesDiallelOAs)) { ReadRDIters[[ii]] = (1:NROW(BayesDiallelOAs[[1]]$MeanVals$ReadTable))}       
ProcessDiallelMSE(TrueParamVec, GriffingOAs, ReadGIters,
     BayesDiallelOAs, ReadRDIters, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb,
     MyLowCI = .025, MyUpCI = .975, PredDiff)
     
GiveTargetGroups = c("aj", "motherj", "dominancej", "Gender:motherj");
dir.create(DirSave, showWarnings=FALSE);
WPT <- WriteParamFightTable(GriffingOAs, BayesDiallelOAs, FileName=paste(DirSave, "//Table1.tex", sep=""),
  MedianSigLevel = .01, MIPSigLevel=.2,
  SubSetIters = NULL, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds = NULL,
  GiveTargetGroups = GiveTargetGroups, roundTo = 3, PredDiff = TRUE)
WPT <- WriteParamFightTable(GriffingOAs, BayesDiallelOAs, FileName=paste(DirSave, "//Table1a.tex", sep=""),
  MedianSigLevel = .01, MIPSigLevel=.2,
  SubSetIters = NULL, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds = NULL,
  GiveTargetGroups = GiveTargetGroups, roundTo = 3, PredDiff = TRUE, KeepGAs = c(1,2), KeepRDs = c(1,2,3) )
WPT <- WriteParamFightTable(GriffingOAs, BayesDiallelOAs, FileName=paste(DirSave, "//Table1b.tex", sep=""),
  MedianSigLevel = .01, MIPSigLevel=.2,
  SubSetIters = NULL, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds = NULL,
  GiveTargetGroups = GiveTargetGroups, roundTo = 3, PredDiff = TRUE, KeepGAs = NULL, KeepRDs = c(4,5,6,7) )
 
 
 TauVOrdered = c("Mu", "Gender:Av", "BetaHybrid:Av", 
  "aj", "motherj", "Gendermotherj", "dominancej", "GenderASymCrossjkDkj",  "BetaHybrid:Gender:Av","Genderaj", "Genderdominancej",
  "SymCrossjk", "GenderSymCrossjk", "ASymCrossjkDkj")
RealVec= c(AFDS$ADiallelOb$Beta[c(1,2,3)], sd(ajVector), sd(motherjVector),
  sd(GendermotherjVector), sd(dominancejVector), sd(GenderASymCrossjkDkjVector), 0, 0, 0, 0, 0, 0);
names(RealVec) = TauVOrdered;
  DoAbs = FALSE;
  
  PlotAllTauPlots(subfraction = 1, DoLog = FALSE,
  DoTauLog = FALSE, DoBayesFactorLog = TRUE, DoMIPLog = FALSE, SaveDir = DirSave,
  FileName = "AllTauPlots.pdf", ExperimentName = "First Fixed Sims",
  TauVOrdered = TauVOrdered, RealVec = RealVec, DoAbs = FALSE
  );
  
  TauVOrdered = c("Gender:Av", "BetaHybrid:Av", 
  "aj", "motherj", "Gendermotherj", "dominancej", "GenderASymCrossjkDkj",  "BetaHybrid:Gender:Av","Genderaj", "Genderdominancej",
  "SymCrossjk", "GenderSymCrossjk", "ASymCrossjkDkj")
RealVec= c(AFDS$ADiallelOb$Beta[c(2,3)], sd(ajVector), sd(motherjVector),
  sd(GendermotherjVector), sd(dominancejVector), sd(GenderASymCrossjkDkjVector), 0, 0, 0, 0, 0, 0);
names(RealVec) = TauVOrdered;
  DoAbs = FALSE;
  
  PlotAllTauPlots(subfraction = 1, DoLog = FALSE,
  DoTauLog = FALSE, DoBayesFactorLog = TRUE, DoMIPLog = FALSE, SaveDir = DirSave,
  FileName = "AllTauPlots.pdf", ExperimentName = "First Fixed Sims",
  TauVOrdered = TauVOrdered, RealVec = RealVec, DoAbs = FALSE
  );

############ 
  dir.create(DirSave, showWarnings=FALSE);
TauVOrdered = c("Gender:Av", "BetaHybrid:Av", "BetaHybrid:Gender:Av",
  "aj", "motherj", "Gendermotherj", "dominancej", "Genderaj", "Genderdominancej",
  "SymCrossjk", "GenderSymCrossjk", "ASymCrossjkDkj", "GenderASymCrossjkDkj")
RealVec= MakeRealVec(TauVOrdered);
names(RealVec) = TauVOrdered;
  DoAbs = FALSE;
  
  PlotAllTauPlots(subfraction = 1, DoLog = FALSE,
  DoTauLog = FALSE, DoBayesFactorLog = TRUE, DoMIPLog = FALSE, SaveDir = DirSave,
  FileName = "AllTauPlots.pdf", ExperimentName = "First Fixed Sims",
  TauVOrdered = TauVOrdered, RealVec = RealVec, DoAbs = FALSE
  );


  

MakeOAs();
dir.create(paste(DropHome, "DiallelWork//WillSimBSep27/", sep=""), showWarnings=FALSE);
WPT <- WriteParamFightTable(GriffingOAs, BayesDiallelOAs, FileName="/Users/alenarc/Dropbox/DiallelWork//WillSimBSep27//Table1.tex",
  MedianSigLevel = .01, MIPSigLevel=.2,
  SubSetIters = NULL, TargetRDTable = TargetRDTable, TargetRDProb = TargetRDProb, HitSimPreds = NULL,
  GiveTargetGroups = 0, roundTo = 3, PredDiff = TRUE)
  

  
TauVOrdered = c("aj", "motherj", "Gendermotherj", "dominancej", "GenderASymCrossjkDkj", "Genderaj", "Genderdominancej", "SymCrossjk", "GenderSymCrossjk", "ASymCrossjkDkj")
PlotTauV(BayesDiallelOAs[[1]], subfraction=1,DoLog = FALSE,
  TauVOrdered = TauVOrdered);
  PlotTauV(BayesDiallelOAs[[1]], subfraction=1,DoLog = FALSE,
  TauVOrdered = NULL)

DirSave = paste(DropHome, "/DiallelWork//WillSimBSept27/", sep="");
dir.create(DirSave, showWarnings=FALSE);
TauVOrdered = c("Mu", "Gender:Av", "BetaHybrid:Av", 
  "aj", "motherj", "Gendermotherj", "dominancej", "GenderASymCrossjkDkj",  "BetaHybrid:Gender:Av","Genderaj", "Genderdominancej",
  "SymCrossjk", "GenderSymCrossjk", "ASymCrossjkDkj")
RealVec= c(AFDS$ADiallelOb$Beta[c(1,2,3)], sd(ajVector), sd(motherjVector),
  sd(GendermotherjVector), sd(dominancejVector), sd(GenderASymCrossjkDkjVector), 0, 0, 0, 0, 0, 0);
names(RealVec) = TauVOrdered;
  DoAbs = FALSE;
  
  PlotAllTauPlots(subfraction = 1, DoLog = FALSE,
  DoTauLog = FALSE, DoBayesFactorLog = TRUE, DoMIPLog = FALSE, SaveDir = DirSave,
  FileName = "AllTauPlots.pdf", ExperimentName = "First Fixed Sims",
  TauVOrdered = TauVOrdered, RealVec = RealVec, DoAbs = FALSE
  );
  
  TauVOrdered = c("Gender:Av", "BetaHybrid:Av", 
  "aj", "motherj", "Gendermotherj", "dominancej", "GenderASymCrossjkDkj",  "BetaHybrid:Gender:Av","Genderaj", "Genderdominancej",
  "SymCrossjk", "GenderSymCrossjk", "ASymCrossjkDkj")
RealVec= c(AFDS$ADiallelOb$Beta[c(2,3)], sd(ajVector), sd(motherjVector),
  sd(GendermotherjVector), sd(dominancejVector), sd(GenderASymCrossjkDkjVector), 0, 0, 0, 0, 0, 0);
names(RealVec) = TauVOrdered;
  DoAbs = FALSE;
  
  PlotAllTauPlots(subfraction = 1, DoLog = FALSE,
  DoTauLog = FALSE, DoBayesFactorLog = TRUE, DoMIPLog = FALSE, SaveDir = DirSave,
  FileName = "AllTauPlots.pdf", ExperimentName = "First Fixed Sims",
  TauVOrdered = TauVOrdered, RealVec = RealVec, DoAbs = FALSE
  );
  }

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
