\name{Models}
\alias{ModelsFile}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Input Model Structure}
\description{
  Models are entered into the diallel analyzer as either a
   vector of character strings (each string a model), or a file where
   every line is another character string representing a model.  There
   are multiple possibilities for model entry, descriptions are provided.
   Models include: additive effect, maternal contribution,
     inbreed effects (general and specific strain),
     and specific cross effects (symmetric and asymmetric), plus all
     sex differentials contained, as well as degrees of freedom for
     a "t-distributed" error process if desired.  See how function \code{\link{ReadModels}}
     converts text of this format into Model Flags useful for \code{\link{BayesDiallelHiddenDetails}}.
}
\usage{
 data(PiximusData);  ls(); Example.Piximus.Models
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{a}{Additive effect, strain specific, also coded "additive"}
  \item{a_s}{Additive effect + sex-specific additive effect differential, also coded "s","as", "gender-additive"}
  \item{B}{General Inbreeding effect, also coded as "inbred"}
  \item{B_s}{General Inbreeding + sex-specific general inbreeding, also coded as "BS", "Bs", "gender-inbred"}
  \item{b}{Strain specific inbreeding effect, also coded as "inbred-strain"}
  \item{b_s}{Strain specific inbreeding effect + gender-strain specific.  Also coded as "gender-inbred-strain"}
  \item{m}{maternal strain specific differential effect, also coded "mother"}
  \item{m_s}{maternal strain specific differential effect + sex-specific maternal, also coded "m_s", "gender-mother"}
  \item{S}{Overall sex effect, also coded "sex"}
  \item{v}{Symmetric-(jk or kj) specific effect, also coded "symmetric"}
  \item{v_s}{Symmetric-(jk or kj) effect + sex-specific version, also coded "sex-symmetric"}
  \item{w}{asymmetric-(+jk but -kj) specific effect of combined breeding, also coded "asymmetric"}
  \item{w_s}{asymmetric-(+jk but -kj) specific effect of combined breeding + sex specific effects, also coded "sex-asymmetric"}
  \item{df}{degrees of freedom of t-noise if applicable.  Answers like "df_6" or "df6" provide t-noise of six degrees of freedom}
  \item{fullu}{Forces the full un-sexed model}
  \item{fulls}{Forces the full sex-ed model.}
  }
\details{
 These model strings (for example, "BSadv" or "B,a_s,d_s,v" or "fulls, df_6") are used to set
   flags in specific "DiallelOb" objects of the overal "FullDiallelAnalyze" structure.
   
   
  The Full Diallel Analyze structure will contain a list called "AllDiallelObs", and
  with "FullDiallelAnalyzeObject$AllDiallelObs[[ii]]", will contain the "ii^th" model
  from the ModelsList.  The flags inherent in this AllDiallelObs will be
  ".ajmodel", ".MotherModel", ".CrossModel", ".SexModel", ".BetaInbredLevel",
    and ".BetaInbredTimesSex".  Details below
        
  DiallelObs are created using "SetupAnalysisMethods"  on a FullDiallel Object
  
  
#### The following is unnecessary documentation of internal model coding structure.
# ajLevel:     \cr                  
##  0 if no Aj terms, 1 if        \cr
## MotherModel: MotherModel = 0 if no mother effect to study, \cr
##     0 if no Mother Effect    \cr
##     1 if j dependent           \cr
##     2 if so complicated as to make Thetaij discrete from Thetaji;\cr

\cr
## Cross Epistatsis Models          \cr
##   0 For No Cross Terms             \cr
##   1 For dominance Hybridj model\cr
##   2 For all pooled\cr
##   3 for pool Theta_ij sim N(0, taui^2i * tauj^2)\cr
##   4 for pool Theta_ij sim N(0, taui^2 + tauj^2);\cr
##   5 for All Pooled with Counter Cross terms\cr
##   6 for pool Theta_ij sim N(0, taui^2i * tauj^2), counter cross\cr
##   7 for pool Theta_ij sim N(0, taui^2i + tauj^2), counter cross\cr
##   8 for All Pooled, ij and ji all separate\cr
##   9 for separate ij and ji both N(0, taui^2i * tauj^2)\cr
##   10 for separate ij and ji both N(0, taui^2i + tauj^2)\cr

\cr
## Beta Hybrid Level\cr
## 0 No Beta Hybrid\cr
## 1 BetaHybrid ~ N(0, tauD);\cr
## 2 BetaHybridj ~ N(0, tauDj);  tauDj ~ InvGamma\cr
## 3 BetaHybridj ~ N(BetaHybrid, tauDj), tauDj ~InvGamma, BetaHybrid ~ tauD; \cr

\cr
## Sex Models\cr
##  0 for No Sex\cr
##  1 for Additive Only Sex\cr
##  2 for Cross Sex aj\cr
##  3 for Cross Sex aj Cross Sex mj\cr
##  4 for Cross Sex aj Cross Sex dj\cr
##  5 for Cross Sex aj Cross Sex mj, Cross taui tauj\cr

\cr
## BetaHybrid * Sex \cr
##  0 No BetaHybrid * Sex\cr
##  1 Else \cr
##BetaInbredTimesSex = 1;\cr


}
\value{
    Object  
     \item{DiallelOb}{Returns R.oo DiallelOb designed for that model}
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{ FullDiallelAnalyze, FullDiallelSimulate, DiallelOb, DiallelAnalyzer}
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
try(require(BayesDiallelGraphics, warn.conflicts=FALSE, quietly=TRUE));

## Trial File


data(PiximusData);

print(Example.Piximus.Models);
ReadModels(MyLines = Example.Piximus.Models);

ReadModels(MyLines="Fulls");
ReadModels(MyLines="Fullu");
ReadModels(MyLines=c("Bs, as, vs, w",
             "bsasvw", "am", "av_sw") );
##tauPriorFile <- read.table(paste(AHome, "/tauPriorFile.txt", sep=""), sep="");
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info);
   
## Get a summary of the Coda Chains for first Diallel Model
summary(AFD$AllDiallelObs[[1]]$CodaChains)
plot(AFD$AllDiallelObs[[1]]$CodaChains);
plot.hpd(AFD$AllDiallelObs[[1]]$CodaChains);

ADO = AFD$AllDiallelObs[[1]];

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
    
## To use plot.hpd, one needs package BayesDiallelGraphics                                      
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot.corr(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

##SimmedGrid = GiveGrid(AFD);
## SimmedGrid only works if you have real values!

par(mfrow=c(1,2));
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior", main="Females")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 0,], phenotype = "Mean Posterior", main="Males")

TwoDrawObject(DT1=MyPostSum[MyPostSum$"Female/Male" == 1,], DT2=MyPostSum[MyPostSum$"Female/Male" == 0,], data.range=NULL,
  phenotype="Mean Posterior",
  phenotype1 = NULL, phenotype2 = NULL, strain.names=unique(as.vector(unlist(Piximus.Data[,1:2]))),
  main1 = "(Females)", main2="(Males)", cols = NULL, 
  col.smoothness = 2, npt = 6, roundDP = 2, HeadTitle = "Piximus Data")
  
DT1=MyPostSum[MyPostSum$"Female/Male" == 1,]; DT2=MyPostSum[MyPostSum$"Female/Male" == 0,]; data.range=NULL;
  phenotype="Mean Posterior";
  phenotype1 = NULL; phenotype2 = NULL; strain.names=unique(as.vector(unlist(Piximus.Data[,1:2])));
  main1 = "(Females)"; main2="(Males)"; cols = NULL; 
  col.smoothness = 2; npt = 6; roundDP = 2; HeadTitle = "Piximus Data"  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
