\name{OneDiallelPlot}
\alias{OneDiallelPlot}
\alias{OneDiallelPlot}
\alias{OnePlot}
\alias{oneplot}
\alias{onediallelplot}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Plot Single Diallel Checkerboard With Scale.}
\description{
  A function a of \code{\link{DiallelOb}} Object.  
  After the chains are run, this takes posterior estimates for all  j,k,S positions of the Diallel and then plots side by side checkerboard plots.
}
\usage{
 OneDiallelPlot(this, AFD = NULL,
  SexlessPlot=TRUE, PlotExpected=TRUE, PlotObserved=FALSE,
  FemalePlot = TRUE,
  mainT = NULL, HeadTitle = NULL, data.range=NULL,
  npt = 6, roundDP = 2, cols = NULL, col.smoothness = 200, 
  LabelLeft=FALSE, LabelTop=FALSE, show.strain.names=TRUE, 
  show.strain.coords=TRUE, new.strain.names = "", ...);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{this}{Member object \code{\link{DiallelOb}}}
  \item{AFD}{\code{\link{FullDialleAnalyze}} object}
  \item{SexlessPlot}{Flag, if true collapses the sexes together}
  \item{PlotExpected}{Flag, if true(default), plots expected or predicted values for checkerboard}
  \item{PlotObserved}{Flag, if PlotExpected is FALSE and PlotObserved is TRUE, plots only observed values}
  \item{FemalePlot}{Flag, if true (And SexlessPlot is FALSE) plots Females only}
  \item{mainT}{A text box checkerboard plot}
  \item{HeadTitle}{Overall head title}
  \item{data.range}{User inputted data.range, else, found in tables}
  \item{npt}{Number of tickmarks to place in the legend at bottom of plot.}
  \item{roundDP}{Round the tickmark values at the legend at bottom of the plot.}
  \item{cols}{Pre-inserted colors for different values (default is grayscale)}
  \item{col.smoothness}{Smoothing criterion for grayscale darkness colors}
  \item{LabelLeft}{Logical, whether to show strain names at left of 
    table or not (bottom is on default if show.strain.names=TRUE)}
  \item{LabelTop}{Logical, whether to show strain names on top of 
    table or not (bottom is on default if show.strain.names=TRUE)}
  \item{show.strain.names}{Logical, Whether to label table with strain.names}
  \item{show.strain.coords}{Logical, Whether to label table with strain coordinates, 
    i.e. 1,2,...}
  \item{new.strain.names}{vector of strings.  If you want a new label of strain names for positions 1:numj
   give these labels.  Note, see \code{AFD$strain.map} to see what the current mapping is.  It is the users
   duty to give strain.names in the same order that reflects \code{strain.map}}

}
\details{
  DiallelOb has several default functions including "copy" and "finalize" for 
  deleting.  
  
  Uses function \code{\link{draw.diallel}}.

}
\value{
  Just plots two plots.
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{FullDiallel}}, \code{\link{DiallelAnalyzer}},
  \code{\link{TwoDiallelPlot}}}
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
##############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = "SaveAFDBackUpPiximusData.RData");   

## Load pregenerated AFD from backup
if (FALSE) {
  library(R.oo);  library(BayesDiallel);
  load("SaveAFDBackUpPiximusData.RData");
}
   
## Now make those TwoDiallelPlots!
## Plot Females, observed versus fitted
AFD$AllDiallelObs[[1]]$TwoDiallelPlot(FemalePlot = TRUE, MaleAgainstFemale = FALSE);

## Plot Males, observed versus fitted
AFD$AllDiallelObs[[1]]$TwoDiallelPlot(FemalePlot = FALSE, MaleAgainstFemale = FALSE);

## Plot Male Predicted versus Female Predicted.
AFD$AllDiallelObs[[1]]$TwoDiallelPlot(FemalePlot = TRUE, MaleAgainstFemale = TRUE);

## Plot Male Observed versus Female Observed.
AFD$AllDiallelObs[[1]]$TwoDiallelPlot(PlotOnlyObserved = TRUE);

## Plot Male Observed versus Female Observed.
AFD$AllDiallelObs[[1]]$OneDiallelPlot(PlotOnlyObserved = TRUE, FemalePlot=TRUE,
  new.strain.names=c("A", "B", "C", "D", "E", "F", "G", "HHH"));


## Plot Observed versus Expected: Sexes combined.
AFD$AllDiallelObs[[1]]$TwoDiallelPlot(PlotObservedVersusExpectedAll = TRUE);  


## Plot Expected Only in One Diallel Plot: Sexes combined.
AFD$AllDiallelObs[[1]]$OneDiallelPlot(SexlessPlot = TRUE, PlotExpected=TRUE, PlotObserved=FALSE);    

## Plot Observed Only in One Diallel Plot: Males only.
AFD$AllDiallelObs[[1]]$OneDiallelPlot(SexlessPlot = FALSE, 
  FemalePlot = FALSE, PlotExpected=FALSE, PlotObserved=TRUE);    
  
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}

## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)

#######################################################3
## Setup Analysis
##
## Allocate AFD : FullDiallelAnalyze Object
AFD  <- FullDiallelAnalyze(Y = Y, 
      SexVector = SexInputVec,
      Listjk = ListjkInput, dfStart = .5, mStart = .5,
      InFixedEffects = AFDS$FixedEffects,
      InRandomEffects = ARandomEffects);

## Assemble a List of models to be considered by AFD
##  This is a k * 6 matrix;
##
## In our case, let k = 1, and supply AFD the "true" simulation model 
ExampleModelsList = matrix(c(ajModel, MotherModel, CrossModel,
      SexModel, BetaInbredLevel, BetaInbredTimesSex), 1, 6);
      Sigma = 1;  
      names(Sigma) = "SigmaAll";

## Setup Analysis Methods for AFD, including allocating for MCMC chains
## "Sigma" is starting Sigma anticipated noise value
SetupAnalysisMethods(AFD, ExampleModelsList, Sigma = Sigma,
      numChains = 3, lengthChains = 2000);

#######################################################
## This will run the MCMC sampler on Model of choice
t1 = proc.time();
  RunChains(AFD);
t2 = proc.time();                  
print(t2-t1); ## Print Run time

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb corresponding to our model

plot(ADO$cent.chains);  ## Plot the centered version of DiallelOb's chains;
plot(ADO$raw.chains);  ## Plot the raw version of DiallelOb's chains;

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
