\name{PlotHPD}
\alias{PlotHPD}
\alias{PlotHPD}
\alias{HPDPlot}
\alias{hpdplot}
\alias{HpdPlot}
\alias{plothpd}
\alias{PlotHpd}
\alias{plotHpd}
\alias{plotHPD}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Plot HPD interval analysis.}
\description{
  A function a of \code{\link{DiallelOb}} Object.  
  After the chains are run, this plot HPD intervals for the plot with \code{strain.map} based names.
}
\usage{
 PlotHPD(this, DoCentered = TRUE, DoRaw = FALSE,
  wanted = NULL, prob.wide=.95, prob.narrow = .5,
  MapChainNames=NULL, UseMapChainNames1=FALSE,xlims=NULL, 
  EvenXlims = TRUE, name.margin=NULL, title.margin=NULL,  bottom.margin=NULL,
  right.margin=NULL, plt.left=NULL, plt.right=NULL, plt.title=NULL,
  plt.bottom=NULL, ylab="", name.line=NULL, title.line=NULL, 
  bottom.line=NULL, right.line=NULL, DoMu = TRUE, HPDWidths = NULL,
  plt.lefts = NULL, ...);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{this}{Member object \code{\link{DiallelOb}} (or owning object of \code{\link{FullDiallelAnalyze}} }
  \item{DoCentered}{Logical Flag, Whether to Plot based upon \code{centered.chains} or \code{raw.chains}}
  \item{DoRaw}{Logical Flag, Whether to Plot based upon \code{centered.chains} or \code{raw.chains}}
  \item{wanted}{Whether to only plot a subset of variables.  Can be just name of random/fixed effects one desires.
    Default is all parameters.  If wanted is a list object, then HPD plot is split into panes.}
  \item{prob.wide}{Width of widest confidence interval (thin drawn line).}
  \item{prob.narrow}{Width of narrowest confidence interval plotted (thic drawn line).}
  \item{MapChainNames}{If the user desires to rename names of variables in Diallel to more intuitive names set this as a keymap}
  \item{UseMapChainNames1}{MapChainNames1 is a default mapping for MapChainNames, set this flag to \code{TRUE} to use it. }
  \item{UseDefaultWanted1}{Flag whether to use a certain "default wanted" list for HPD plots known as DefaultWanted1. }
  \item{xlims}{List of limits for x axis for HPD plots, if wanted is multiple sections, should
    be a matrix of more height length of wanted and width of two for min and max limits}
  \item{EvenXlims}{Whether to try to supply even xlims to all members of multiple wanted list}
  \item{title.margin}{Set a different title.margin, alternatively, plt.title does this work}
  \item{bottom.margin}{Set a different bottom.margin, alternatively, plt.bottom does this work}
  \item{title.line}{Set a different title.line}
  \item{name.line}{Set a different name.line}
  \item{DoMu}{Logical Flah whether to include "Mu" intercept estimate}
  \item{HPDWidths}{Supply a columns widths for each of the columns of multicolumn HPD plot.}
  \item{plt.lefts}{Supply automatic left label widths for each of the columns of a multicolumn HPD plot.}
}
\details{
  DiallelOb has several default functions including "copy" and "finalize" for 
  deleting.   Uses an accepts all inputs to function \code{plot.hpd}/
  

}
\value{
  Just plots a plot.
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{FullDiallel}}, \code{\link{DiallelAnalyzer}} }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
##############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = "SaveAFDBackUpPiximusData.RData");   
  
## Load pregenerated AFD from backup
if (FALSE) {
  library(R.oo);  library(BayesDiallel);
  load("SaveAFDBackUpPiximusData.RData");
}
 
## Now make those HPD Plots!
AFD$AllDiallelObs[[1]]$PlotHPD();
AFD$AllDiallelObs[[1]]$PlotHPD(wanted = c("aj", "Gender:aj"), name.margin = 7, title.margin = 5,
  xlab="Confidence intervals");
AFD$AllDiallelObs[[1]]$PlotHPD(wanted = c("aj", "Gender:aj"), name.margin = 7, title.margin = 5,
  xlab="Confidence intervals", xlims=c(-2,2) );
  
  
AFD$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE,
  xlab="Confidence intervals", EvenXlims=TRUE, DoMu = FALSE);

AFD$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE,
  xlab="Confidence intervals", EvenXlims=TRUE, HPDWidths = c(1,1,1,1), DoMu = FALSE);
  
AFD$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE,
  xlab="Confidence intervals", EvenXlims=TRUE, HPDWidths = c(1.2,1,1.2,1), 
  plt.lefts=c(.3,.1,.3,.1), DoMu = FALSE);

## This version removes Mean intercept, also uses "UseDefaultWanted1" new
##   Beta names (including "Female" for "gender")
AFD$AllDiallelObs[[1]]$PlotHPD(UseDefaultWanted1 = TRUE,
  UseMapChainNames1 = TRUE,
  xlab="Confidence intervals", EvenXlims=TRUE, DoMu=FALSE);
    
    
  
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
