\name{PosteriorPredSummary}
\alias{PosteriorPredSummary}
\alias{PosteriorPredSummary}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Diallel Object}
\description{
  A function a of \code{\link{DiallelOb}} Object.  
  After the chains are run, this gives posterior estimates for all  j,k,S positions of the Diallel, regardless of whether
  these squares were observed or unobserved.  
}
\usage{
PosteriorPredSummary(this, AFD = NULL,
  burnin = -1, thin = 1, keep = FALSE, plotdensity=FALSE,
  WithNoise = FALSE, ...) 
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{this}{Member object \code{\link{DiallelOb}}}
  \item{burnin}{Integer amount of Gibbs sample analysis fitting chains to remove}
  \item{thin}{Integer how many iterations to skip every step when making posterior estimates}
  \item{keep}{Flag that when TRUE, keeps the posterior prediciton MCMC chains in the DiallelOb after analysis}
  \item{plotdensity}{Flag whether to generate a density plot.}
  \item{WithNoise}{Flag that when FALSE, the statistics refer to posterior predictive density of the mean
   at each point in Diallel.  When Noise is added, the statistics are a posterior predictive 
     density for future datasets (which would have the noise model as well as uncertainty)}

}
\details{
  DiallelOb has several default functions including "copy" and "finalize" for 
  deleting.  
  
  PosteriorPredSummary is only one member of the \code{\link{DiallelOb}} functions.
  
  Note that function \code{PosteriorPredSummary} is not often needed, because two default getter aliases
  already exist in \code{\link{DiallelOb}}.  
  Accessing members \code{PosteriorPredictionsMeanSummary} and 
  \code{PosteriorPredictionsErrorSummary} is often the only thing one need do.

}
\value{

    When "keep=TRUE" flag is set, this function saves to the DiallelOb an element
      \code{PostKeeper}.  \code{PostKeeper} is a list containing three items.
      
      = list(FakeListjk = FakeListjk, 
     FakeSexVector = FakeSexVector, FakeCoda = FakeCoda)
     
    \item{FakeListjk}{A list of positions in the full Diallel}
    \item{FakeSexVector}{An accompany list of Diallel Sexes for those positions.}
    \item{FakeCoda}{The Posterior prediction mcmc chains as generated by this function to create summary statistics} 
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{FullDiallel}}, \code{\link{DiallelAnalyzer}},
  \code{\link{SpeculativeGenesSimulation}} }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
##############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   SaveAFDFile = "SaveAFDBackUpPiximusData.RData");   
   
## Posteirior Prediction Mean Summary Chains;
AFD$AllDiallelObs[[AFD$OnObs]]$PosteriorPredictionMeanSummary;  

 ## Posterior Predition Summary Chains (the error is added)
AFD$AllDiallelObs[[AFD$OnObs]]$PosteriorPredictionsErrorSummary; 
  
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}

## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)

#######################################################3
## Setup Analysis
##
## Allocate AFD : FullDiallelAnalyze Object
AFD  <- FullDiallelAnalyze(Y = Y, 
      SexVector = SexInputVec,
      Listjk = ListjkInput, dfStart = .5, mStart = .5,
      InFixedEffects = AFDS$FixedEffects,
      InRandomEffects = ARandomEffects);

## Assemble a List of models to be considered by AFD
##  This is a k * 6 matrix;
##
## In our case, let k = 1, and supply AFD the "true" simulation model 
ExampleModelsList = matrix(c(ajModel, MotherModel, CrossModel,
      SexModel, BetaInbredLevel, BetaInbredTimesSex), 1, 6);
      Sigma = 1;  
      names(Sigma) = "SigmaAll";

## Setup Analysis Methods for AFD, including allocating for MCMC chains
## "Sigma" is starting Sigma anticipated noise value
SetupAnalysisMethods(AFD, ExampleModelsList, Sigma = Sigma,
      numChains = 3, lengthChains = 2000);

#######################################################
## This will run the MCMC sampler on Model of choice
t1 = proc.time();
  RunChains(AFD);
t2 = proc.time();                  
print(t2-t1); ## Print Run time

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb corresponding to our model

plot(ADO$cent.chains);  ## Plot the centered version of DiallelOb's chains;
plot(ADO$raw.chains);  ## Plot the raw version of DiallelOb's chains;

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
