\name{PosteriorPredictionVariability}
\alias{PosteriorPredictionVariability}
\alias{SpeculativeVariability}
\alias{KDim}
\alias{SpeculativeGenes}
\alias{PosteriorVariabilitySim}
\alias{SpeculativeGenesSimulation}
\alias{SpeculativeGenesBackCrossSimulation}
\alias{SpeculativeWeightGenesSimulation}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Diallel Object}
\description{
  A function a of \code{\link{DiallelOb}} Object.  
  
  After the chains are run \code{\link{PosteriorPredSummary}} 
  gives posterior estimates for all  j,k,S positions of the Diallel, 
  regardless of whether
  these squares were observed or unobserved.  
  
  This function \code{\link{SpeculativeGenesSimulation}} then allows one to speculate,
   for F2 or further populations, that are crosses of any two of the strains in the diallel,
   what would be the expected variability for a multi-locus model of genes?
   
  Function \code{\link{SpeculativeGenesBackCrossSimulation}} speculates the diversity 
  of a backcross population.  This would be the product of an "F2+" jxJ hybrid with
  an inbred of fully jxj. While one of the parental contributions is fully deterined, 
  the other half will be randomized. 
  
  Function \code{\link{SpeculativeWeightGenesSimulation}} is slightly more complex.
  Instead of assuming a Gaussian distribution of alleles, the user specifies a new  input
  "UseMyWeightFunction" which is a function generating a list of non-negative w_i  which sum to 1.
  This weight function is w_i=1/KDim for Gaussian distribution, but can be replaced with T-distribution
  and spike distributions.  A funtion "SigmakKFunction" should specify the expected inheritance distribution of 
  covariances between nearby/far-away genes.
}
\usage{
SpeculativeGenesSimulation(this,
  UseFixed = FALSE, KDim = 1:10, 
  pMix = .5, barSigmakK=0, barSigmakKSq=0, ...)
  
SpeculativeGenesBackCrossSimulation(this,
  UseFixed = FALSE, KDim = 1:10, 
  pMix = .5, barSigmakK=0, ...)
  
SpeculativeWeightGenesSimulation(this,
  UseFixed = FALSE, KDim = 1:10, 
  pMix = .5, SigmakK=0, SexMix=.5,UseMyWeightFunction =DefaultUseMyWeightFunction,
  SigmakKFunction = NULL, ListSigmakK=NULL,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{this}{Member object \code{\link{DiallelOb}}}
  \item{UseFixed}{Flag whether to include immobile fixed effects in variability}
  \item{KDim}{Speculative counts of genes 1,2 or many.  Can be supplied}
  \item{pMix}{A calculation of the balance between "j" alleles and "J" alleles in
   the speculative population.  Most often pMix=.5 is recommended}
  \item{barSigmakK}{Average correlation between the KDim genes expected in the cross.
    If genes are on different chromosomes, or if enough LD exists between genes,
      one can reasonably expect there to be no correlation between genes. 
  }
  \item{barSigmakK}{Average squared correlation between the KDim genes expected in the cross.
    If genes are on different chromosomes, or if enough LD exists between genes,
      one can reasonably expect there to be no correlation between genes and thus
      the squared correlation is also zero. 
  }
  \item{UseMyWeightFunction}{A function which generates weights for 
  \code{\link{SpeculativeWeightGenesSimulation}}  
  }
  \item{SigmakKFunction}{A function which generates inheritance correlation matrices
     for 
  \code{\link{SpeculativeWeightGenesSimulation}.  It should take a dimension, say KDim=5 and
  create a KDim * KDim symmetric positive definite matrix.}  
  }
}
\details{
  DiallelOb has several default functions including "copy" and "finalize" for 
  deleting.  
  
  SpeculativeGenesSimulation is only one member of the \code{\link{DiallelOb}} functions.
  
  Note that \code{SpeculativeGenesSimulation} relies on function \code{\link{PosteriorPredSummary}}.

}
\value{

    Returns a list with 8 objects
     
      
     
    \item{VarSummary}{A coda package summary of expected variance of strain pairings,
      There are as many summaries as speculative numbers of genes KDim were given}
    \item{NoiseSummary}{The summary of the environmental noise variability}
    \item{VarChain}{The MCMC chains used to generate VarSummary}
    \item{NoiseChain}{The MCMC chain used to generate NoiseSummary}
    \item{BetajjChain}{An MCMC chain where each column refers to the "inbred" jxj
      specimen measure used for this speculative variability.  Number of columns
      equal the number of rows for VarSummary. 
    }
    \item{BetaJJChain}{An MCMC chain where each column refers to the "inbred" JxJ
      specimen measure used for this speculative variability.   
    }
    \item{BetajJChain}{An MCMC chain where each column refers to the F1 hybrid jxJ
      specimen measure used for this speculative variability.   
    }
    \item{BetaJjChain}{An MCMC chain where each column refers to the F1 hybrid Jxj
      specimen measure used for this speculative variability.   
    }
    \item{VarianceWithSexSummary}{This is a summary of the total variance of a population with
      sexes combined.  Sex can be a driving factor in diversity of phenotype, both from a main effect and
      strain-dependent effects.  However, the combined populations of both sexes represents both
      the variation within sexes and the difference in means between both sexes.  This is not always an adequate baseline
      for inspecting genetic effects.  Sex specific efects which will be recounted in "VarSummary"}
    \item{VarianceWithSexChain}{The MCMC chain determining VarianceWithSexSummary}
    
    SpeculativeWeight uses different returns:
    
    \item{ListSumWithCov}{MCMC Summary List, when genes have inheritance correlation}
    \item{ListSumNoCov}{MCMC Summary List, when there is no correlation between genes} 
    \item{ListVarWithCov}{MCMC Chains, when genes have inheritance correlation}
    \item{ListVarNoCov}{MCMC Chains, when there is no correlation between genes} 
    \item{NoiseSummary}{MCMC Summary, of noise sigma^2}  
    \item{ListVarWithSexWithCovChain}{MCMC Chains, variance of all population including sex variance}  
    \item{ListVarWithSexWithCovSummary}{MCMC Summary, variance of all population including sex variance}  
    \item{ListVarWithSexNoCovSummary}{MCMC Chains, variance of all population including sex variance, no inherited correlation}  
    \item{ListVarWithSexNoCovChain}{MCMC Summary, variance of all population including sex variance, no inherited correlation} 
    \item{MeanChains}{Mean Chains: means of the jj, jJ, Jj, JJ populations}       
    
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{FullDiallel}}, \code{\link{DiallelAnalyzer}} }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
##############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   SaveAFDFile = "SaveAFDBackUpPiximusData.RData");   
   
## Posterior Prediction Mean Summary Chains;
AFD$AllDiallelObs[[AFD$OnObs]]$PosteriorPredictionMeanSummary;  

 ## Posterior Predition Summary Chains (the error is added)
AFD$AllDiallelObs[[AFD$OnObs]]$PosteriorPredictionsErrorSummary; 

## SpeculativeGeneMeausurements
SpecMeasures <- AFD$AllDiallelObs[[1]]$SpeculativeGenesSimulation(
  UseFixed = FALSE, KDim = 1:10, 
  pMix = .5, barSigmakK=0, barSigmakKSq=0)
  
#################################################################################
## This example looks at the crosses that could be performed with 3 genes believed to exist
## We find the jxJ cross with greatest variability and plot the means of the inbred/F1-hybrids
##  as well as a speculative Gaussian density with the variability predicted
##
SpecGenes <- 3;  ## Get the cross with the greatest variability for 3 genes
VarSummary <- SpecMeasures$VarSummary;
BigRow <- (1:NROW(VarSummary[[SpecGenes]][[1]]))[VarSummary[[SpecGenes]][[1]][,1] == max(VarSummary[[SpecGenes]][[1]][,1])];
VarSummary[[SpecGenes]][[1]][BigRow,]  ## display summary of this cross.
NameCross <- rownames(VarSummary[[SpecGenes]][[1]])[BigRow] ## What was this cross?
LSS <- unlist(strsplit(NameCross, ":"));
NewNameCross <- paste("S:", LSS[2], ":j:", AFD$strain.map[AFD$strain.map[,2] == LSS[4],1],
  ":J:",AFD$strain.map[AFD$strain.map[,2] == LSS[6],1], sep=""); 
jjMean <- mean(SpecMeasures$BetajjChain[,BigRow]);
JJMean <- mean(SpecMeasures$BetaJJChain[,BigRow]);
JjMean <- mean(SpecMeasures$BetaJjChain[,BigRow]);
jJMean <- mean(SpecMeasures$BetajJChain[,BigRow]);
MiddleOfCrosses <- mean(c(jjMean, JJMean, JjMean, jJMean));
xlim <- c(min(c(jjMean,JJMean,JjMean,jJMean,MiddleOfCrosses-2.5*sqrt(VarSummary[[SpecGenes]][[1]][BigRow,1]))),
 max(c(jjMean,JJMean,JjMean,jJMean,MiddleOfCrosses+2.5*sqrt(VarSummary[[SpecGenes]][[1]][BigRow,1]))));
xxs <- xlim[1]+(xlim[2]-xlim[1])*(0:1000)/1000;
MyDens <- dnorm(xxs,MiddleOfCrosses,sqrt(VarSummary[[SpecGenes]][[1]][BigRow,1]));
plot(MyDens~xxs, xlim=xlim, main=paste("Maximum Weight Variability Cross under 3 Spec Genes population: ",
  NewNameCross, sep=""),
  ylab="Density", xlab="Phenotype", type="l", lwd=2, col="black", lty=3);
lines(c(jjMean,jjMean),c(0,max(MyDens)), col="red", lwd=2, lty=1);
lines(c(JJMean,JJMean),c(0,max(MyDens)), col="blue", lwd=2, lty=1);
lines(c(jJMean,jJMean),c(0,max(MyDens)), col="purple", lwd=2, lty=1);
lines(c(JjMean,JjMean),c(0,max(MyDens)), col="purple", lwd=2, lty=1);


BigRows <- rep(0, length(VarSummary));
for (ii in 1:length(VarSummary)) {
  BigRows[ii] <- (1:NROW(VarSummary[[ii]]))[VarSummary[[ii]][[1]][,1] == max(VarSummary[[ii]][[1]][,1])];
}

## SpeculativeGeneMeausurements
SpecMeasuresBack <- AFD$AllDiallelObs[[1]]$SpeculativeGenesBackCrossSimulation(
  UseFixed = FALSE, KDim = 1:10, 
  pMix = .5, barSigmakK=0)
  
#################################################################################
## This example looks at the crosses that could be performed with 3 genes believed to exist
## We find the jxJ cross with greatest variability and plot the means of the inbred/F1-hybrids
##  as well as a speculative Gaussian density with the variability predicted
##
SpecGenes <- 3;  ## Get the cross with the greatest variability for 3 genes
VarSummary <- SpecMeasuresBack$VarSummary;
BigRow <- (1:NROW(VarSummary[[SpecGenes]][[1]]))[VarSummary[[SpecGenes]][[1]][,1] == max(VarSummary[[SpecGenes]][[1]][,1])];
VarSummary[[SpecGenes]][[1]][BigRow,]  ## display summary of this cross.
NameCross <- rownames(VarSummary[[SpecGenes]][[1]])[BigRow] ## What was this cross?
LSS <- unlist(strsplit(NameCross, ":"));
NewNameCross <- paste("S:", LSS[2], ":j:", AFD$strain.map[AFD$strain.map[,2] == LSS[4],1],
  ":J:",AFD$strain.map[AFD$strain.map[,2] == LSS[6],1], sep=""); 
jjMean <- mean(SpecMeasuresBack$BetajjChain[,BigRow]);
jJMean <- mean(SpecMeasuresBack$BetajJChain[,BigRow]);
MiddleOfCrosses <- mean(c(jjMean, jJMean));
xlim <- c(min(c(jjMean,jJMean,MiddleOfCrosses-2.5*sqrt(VarSummary[[SpecGenes]][[1]][BigRow,1]))),
 max(c(jjMean,jJMean,MiddleOfCrosses+2.5*sqrt(VarSummary[[SpecGenes]][[1]][BigRow,1]))));
xxs <- xlim[1]+(xlim[2]-xlim[1])*(0:1000)/1000;
MyDens <- dnorm(xxs,MiddleOfCrosses,sqrt(VarSummary[[SpecGenes]][[1]][BigRow,1]));
plot(MyDens~xxs, xlim=xlim, main=paste("Maximum Weight Variability Cross under 3 Spec Genes population: ",
  NewNameCross, sep=""),
  ylab="Density", xlab="Phenotype", type="l", lwd=2, col="black", lty=3);
lines(c(jjMean,jjMean),c(0,max(MyDens)), col="red", lwd=2, lty=1);
lines(c(jJMean,jJMean),c(0,max(MyDens)), col="purple", lwd=2, lty=1);


## Speculative variance when t-distribution is assumed.
SpecWithWeight <- AFD$AllDiallelObs[[1]]$SpeculativeWeightGenesSimulation(this,
  UseFixed = FALSE, KDim = 1:10, 
  pMix = .5, SigmakK=0, SexMix=.5,UseMyWeightFunction =DefaultUseMyWeightFunction,
  SigmakKFunction = SigmakKSampleFunction, ListSigmakK=NULL)
  
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}

## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)

#######################################################3
## Setup Analysis
##
## Allocate AFD : FullDiallelAnalyze Object
AFD  <- FullDiallelAnalyze(Y = Y, 
      SexVector = SexInputVec,
      Listjk = ListjkInput, dfStart = .5, mStart = .5,
      InFixedEffects = AFDS$FixedEffects,
      InRandomEffects = ARandomEffects);

## Assemble a List of models to be considered by AFD
##  This is a k * 6 matrix;
##
## In our case, let k = 1, and supply AFD the "true" simulation model 
ExampleModelsList = matrix(c(ajModel, MotherModel, CrossModel,
      SexModel, BetaInbredLevel, BetaInbredTimesSex), 1, 6);
      Sigma = 1;  
      names(Sigma) = "SigmaAll";

## Setup Analysis Methods for AFD, including allocating for MCMC chains
## "Sigma" is starting Sigma anticipated noise value
SetupAnalysisMethods(AFD, ExampleModelsList, Sigma = Sigma,
      numChains = 3, lengthChains = 2000);

#######################################################
## This will run the MCMC sampler on Model of choice
t1 = proc.time();
  RunChains(AFD);
t2 = proc.time();                  
print(t2-t1); ## Print Run time

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb corresponding to our model

plot(ADO$cent.chains);  ## Plot the centered version of DiallelOb's chains;
plot(ADO$raw.chains);  ## Plot the raw version of DiallelOb's chains;

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
