\name{PosteriorRSq}
\alias{PosteriorRSq}
\alias{PosteriorRSq}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{PosteriorRSq}
\description{
  PosteriorRSq
  
  This attempts to show amount of variability in the observed
     diallel data using the observed strains and fitted beta components.
    This lies half way between \code{\link{PosteriorDSq}} and \code{\link{PosteriorHSq}} 
     However, these fitted factors interact, so the 
   sum of these RSq components is not 1.0;  
}
\usage{
PosteriorRSq(this="DiallelOb", UseFixed = FALSE,
  ProperRatio = FALSE, AFD = NULL, burnin =1, thin = 1,
  TrueVector = NULL,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{this}{Member object \code{\link{DiallelOb}}}
  \item{burnin}{Integer amount of Gibbs sample analysis fitting chains to remove}
  \item{thin}{Integer how many iterations to skip every step when making posterior estimates}
  \item{UseFixed}{Flag that when TRUE, includes fixed effects as components to HS1q ratio,
    if FALSE, then leaves fixed effects out}
  \item{ProperRatio}{Flag that when TRUE, Uses proper ratio between tau^2 based upon future samples of a J * J diallel,
    if FALSE, then the HSq is composed of a flat ratio (equally weighted) between components}
  \item{AFD}{Parent \code{\link{FullDiallelAnalyze}} object.}
  \item{TrueVector}{If True HSq is known (because true tau^2 is known), this allows differentials to be calculated}
}
\details{
  RSq list object, matrix
  This includes "LPRSq" which is a Coda list for summary values and PointDSq
  
}
\value{
  Returns a list object, LPDSq component is codea chains, PointRSq is based upon point estimate means of RSq,
    and tPointRSq is an estimate of heritability implied by the "TrueVector"
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, 
  \code{\link{FullDiallelSimulate}}, \code{\link{FullDiallel}}, 
  \code{\link{PosteriorDSq}}, \code{\link{PosteriorHSq}}}
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);

## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   SaveAFDFile = "SaveAFDBackUpPiximusData.RData");   
   
## Posteirior Prediction Mean Summary Chains;
AFD$AllDiallelObs[[AFD$OnObs]]$PosteriorPredictionMeanSummary;  

 ## Posterior Predition Summary Chains (the error is added)
AFD$AllDiallelObs[[AFD$OnObs]]$PosteriorPredictionsErrorSummary; 

#################################################################################
##  HSq, RSq, and DSq
##
##   HSq is our future heritability metric, based upon estimated tau^2 parameters,
##    it assumes independence of  future diallels.
##
##   RSq relates the amount of variation in Y explained by a covariate.  Note, that
##    because of correlation between terms, sum of RSq terms can be more or less than
##    total variation of Y.  
##
## Give a table summarizing beliefs in heritability ratios for future independent diallels.
##  expresses residual variance in terms of 
AFD$AllDiallelObs[[1]]$PosteriorRSqTable

## Inner method that calculates that HSq
OtherPosteriorChains = AFD$AllDiallelObs[[1]]$PosteriorRSq(UseFixed=TRUE, ProperRatio=TRUE,
      AFD = AFD, startiter = AFD$burin, thin = AFD$thin)
      
## Now look at RSq as a different metric
summary(AFD$AllDiallelObs[[1]]$PosteriorRSqChains);
   
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}

## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)

#######################################################3
## Setup Analysis
##
## Allocate AFD : FullDiallelAnalyze Object
AFD  <- FullDiallelAnalyze(Y = Y, 
      SexVector = SexInputVec,
      Listjk = ListjkInput, dfStart = .5, mStart = .5,
      InFixedEffects = AFDS$FixedEffects,
      InRandomEffects = ARandomEffects);

## Assemble a List of models to be considered by AFD
##  This is a k * 6 matrix;
##
## In our case, let k = 1, and supply AFD the "true" simulation model 
ExampleModelsList = matrix(c(ajModel, MotherModel, CrossModel,
      SexModel, BetaInbredLevel, BetaInbredTimesSex), 1, 6);
      Sigma = 1;  
      names(Sigma) = "SigmaAll";

## Setup Analysis Methods for AFD, including allocating for MCMC chains
## "Sigma" is starting Sigma anticipated noise value
SetupAnalysisMethods(AFD, ExampleModelsList, Sigma = Sigma,
      numChains = 3, lengthChains = 2000);

#######################################################
## This will run the MCMC sampler on Model of choice
t1 = proc.time();
  RunChains(AFD);
t2 = proc.time();                  
print(t2-t1); ## Print Run time

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb corresponding to our model

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
MXM = AFD$AllDiallelObs[[1]]$CreateMatrixMX();
ReducedChain = MXChain(MXM, AFD$AllDiallelObs[[1]]$CodaChains);
summary(ReducedChain);

}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
