\name{RConstantsList}
\alias{RConstantsList}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{BayesDiallel Package}
\description{
  An R.oo Object used for storing the constants of an R list
    
  Function "RConstantsList"  and Object "FullDiallelAnalyze" are key pieces of
   info
  
  One RConstants list will be the "m" members, and the other will be the "df",
    The priors modelled are inverse-chisquared:
       X ~   m*df / rchisq(df)
}
\usage{
  RConstantsList(StartF = .01, 
    StartRandomEffects = NULL,...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{StartF}{Starting Value}
}
\details{
   Both an "m" and an "df" should be found in
     FullDiallelAnalyze or FullDiallelSimulate Objects
}
\value{
    Object  
     \item{DiallelOb}{Returns S3 class generated}
}
\references{ BayesDiallel Book }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  ""  is a class containing an model, and possibly gibbs chains.
}
\seealso{ FullDiallelAnalyze, FullDiallelSimulate, DiallelOb }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);

TrueModel = c("BSasmvb");
WalkAndReadCrossList(TrueModel)  
## Set the model shape to simulate
  ModelVector = c(1,1,2,1,3,1);
  ModelVector = WalkAndReadCrossList(TrueModel);
  ajModel = ModelVector[1];
  MotherModel = ModelVector[2];
  CrossModel = ModelVector[3];
  BetaInbredLevel = ModelVector[4];
  SexModel = ModelVector[5];
  BetaInbredTimesSex = ModelVector[6];
  IsFixed = 1; IsRandom = 1;
  NumReps = 2 ## Imagine dataset of size 2 times full diallel
  numj = 8 ## 8 strains of mice?
  
  
print("Sampling NonDiallel Random and Fake Effects");
InFixedEffects=NULL; InRandomEffects =NULL;
if (IsFixed || IsRandom) {
   WantLen = (numj *(numj+1)/2 ) *NumReps ;
   InRandomEffects = sample(size=WantLen, 1:5, replace=TRUE);
   InFixedEffects = rnorm(WantLen);
   if (IsFixed == 0) {InFixedEffects=NULL;}
   if (IsRandom == 0) {InRandomEffects=NULL;}
   mRandomEffects = 20;
   dfRandomEffects = 5; tauFixedEffects = 20;
   NumRepetitions=1;
}  else {
   NumRepetitions = NumReps;
}

## Simulate a data set given model of choice
print("Making first Simulation");
### Allocate AFDS Diallel Simulation Object
AFDS <- FullDiallelSimulate(numj = numj, InFixedEffects = InFixedEffects,
 InRandomEffects = InRandomEffects, mRandomEffects = 20,
 dfRandomEffects = 5, tauFixedEffects = 20, YesSex = TRUE, NumRepetitions = NumRepetitions);

## Sample the parameters for Diallel Simulation Object
defaultSimulateParam(AFDS);
if (is.null(InRandomEffects) && is.null(InFixedEffects)) {
  SetupFilledSimulatedListjk(AFDS, NumRepetitions = NumRepetitions);
}


## Set up the model for Diallel Simulation Object
SetupSimulateMethod(AFDS, ajModel = 1, MotherModel = MotherModel, CrossModel =
  CrossModel, SexModel = SexModel, BetaInbredLevel = BetaInbredLevel, 
  BetaInbredTimesSex = BetaInbredTimesSex);
## SimValues now simulates a random Beta vector, tau vector, and observed data Y      
SimValues(AFDS, Sigma = 1);

#############################################################################
###  Begin Analysis
###
### From Simulated Diallel in AFDS, we pull observed data to put in an 
###  Analysis Diallel AFD

Y = AFDS$Y; ListjkInput = AFDS$Listjk;
SexInputVec = AFDS$SexVector

## Pull Out the RandomEffects
ARandomEffects = rep(0, length(AFDS$RandomEffects[,1]));
for (ii in 1:length(AFDS$RandomEffects[1,])) {
  ARandomEffects[AFDS$RandomEffects[,ii] == 1] = ii;
}
if (is.null(InRandomEffects)) { ARandomEfects = NULL; }
##TargetLength=TARGET;
SampleBeta = AFDS$ADiallelOb$Beta;
FileDir = AllDir2;
quantile = .95;
INTIN = 0;

## These are the true simulation parameters from AFDS we would like experiment to learn  
TrueParametersList = c(AFDS$ADiallelOb$Beta, AFDS$ADiallelOb$tau,
    AFDS$ADiallelOb$Sigma)
    
Models =  c(TrueModel, "BSadf3s", "Bvswabsaas");

    
########################################################
##  Call DiallelAnalyze
##
MyDataFileTable = cbind(Y, ListjkInput, SexInputVec, AFDS$FixedEffects, ARandomEffects);
colnames(MyDataFileTable) = c("phenotype", "mother.strain", "father.strain", 
  "sex", colnames(AFDS$FixedEffects), colnames(ARandomEffects));
AFD = DiallelAnalyzer(DataFile = MyDataFileTable, father.strain="father.strain",
 mother.strain="mother.strain", phenotype="phenotype", is.female="is_female",
   SexVector="sex", FixedEffects = colnames(AFDS$FixedEffects),
   RandomEffects = ARandomEffects, 
   Models=Models, Sigma.start = 1;  numChains = 5;  lengthChains=2500,burnin.start = 1,
   DIC.Only=FALSE,  TauFile = NULL);

ADO = AFD$AllDiallelObs[[1]];  ## This is the DiallelOb 
                               ## corresponding to our true model

plot(ADO$CodaChains);  ## Plot the chains to understand
summary(ADO$CodaChains);  ## Get Summary information on a parameters
                                          
plot.hpd(ADO$CodaChains); ## Using Library_diallel_helper, plot chains.

plot(crosscorr(ADO$CodaChains)) ## Look at cross correlations of coda chains


CalcDICC(ADO, AFD) ## DIC value for this Coda Chain  

## This creates a summary table of applicable elements
MySum = summary.table(ADO, burnin = 400, AFD=AFD);

#MyPostSumGivesValues for all the diallel squares of every gender
MyPostSum = PosteriorPredSummary(ADO, AFD, burnin = 400, AFD=AFD, keep = TRUE);

SimmedGrid = GiveGrid(AFDS);
par(mfrow=c(1,2));
draw.diallel(data =SimmedGrid[MyPostSum$"Female/Male" == 1,], phenotype = "Mean.Simulated")
draw.diallel(data = MyPostSum[MyPostSum$"Female/Male" == 1,], phenotype = "Mean Posterior")
  
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
