\name{RenameChains}
\alias{RenameChains}
\alias{RenameChains}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{Function gives a MCMC coda chain thinned and renamed to match strain.map.}
\description{
  A function a of \code{\link{DiallelOb}} Object.  
  After the chains are run, this will Give a Coda object with titles formatted to include the
  name of strains as contained in first column of AFD$strain.map and to have the name
  of effects renamed according to a mapping MapChainNames.
}
\usage{
 PlotHPD(this, DoCentered = TRUE, DoRaw = FALSE,
  wanted = NULL, prob.wide=.95, prob.narrow = .5,
  MapChainNames=NULL, UseMapChainNames1=FALSE,...);
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{this}{Member object \code{\link{DiallelOb}} (or owning object of \code{\link{FullDiallelAnalyze}} }
  \item{DoCentered}{Logical Flag, Whether to Plot based upon \code{centered.chains} or \code{raw.chains}}
  \item{DoRaw}{Logical Flag, Whether to Plot based upon \code{centered.chains} or \code{raw.chains}}
  \item{wanted}{Whether to only plot a subset of variables.  Can be just name of random/fixed effects one desires.
    Default is all parameters.}
  \item{MapChainNames}{If the user desires to rename names of variables in Diallel to more intuitive names set this as a keymap}
  \item{UseMapChainNames1}{MapChainNames1 is a default mapping for MapChainNames, set this flag to \code{TRUE} to use it.
    \code{BayesDiallel:::MapChainNames1} to see the Mapping.}

}
\details{
  DiallelOb has several default functions including "copy" and "finalize" for 
  deleting.   Uses an accepts all inputs to function \code{plot.hpd}/
  
  The owning member of the DiallelOb "AFD" should contain a mapping "\code{strain.map}".
  The first column of this map is the name in character format of the strains used in the study,
    while the right column assigns those strains numeric code.  If \code{AFD$strain.map} is NULL,
    the numeric code is used as seen in \code{AFD$Listjk}.
    
  Furtermore "\code{MapChainNames}" is an encoding that maps the names of Beta coefficient effects
    in te diallel (such as "aj" and "Gender:aj" and "SymCrossjk") in to more descriptive terms of the
    users interest.  An example mapping is seen at \code{BayesDiallel:::MapChainNames1}

}
\value{
  Returns a coda object with necessary coefficients remapped.
}
\references{ Diallel Paper }
\author{ Alan Lenarcic, Will Valdar }
\note{ 
  "DiallelOb"  is a class containing an model, and possibly gibbs chains.
}
\seealso{\code{\link{FullDiallelAnalyze}}, \code{\link{FullDiallelSimulate}}, 
  \code{\link{FullDiallel}}, \code{\link{DiallelAnalyzer}} }
\examples{
## Load in R packages
try(require(lattice, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(coda, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.methodsS3, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(R.oo, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
try(require(corpcor, quietly=TRUE, warn.conflicts=FALSE), silent=TRUE);
require(BayesDiallel, quietly = TRUE, warn.conflicts=FALSE);
require(R.oo, warn.conflicts=FALSE, quietly=TRUE);
   
##############################################################################
## First Example Piximus Diallel Mouse Data.
data(PiximusData);
##  Data is in "Piximus.Data"
##  Models are in "Examples.Piximus.Models"
##  tau.Prior is in "Examples.Piximus.tau.Prior.Info"
####################################
###### Run the Algorithm->
AFD = DiallelAnalyzer(data = Piximus.Data, father.strain="father.strain.name",
   mother.strain="mother.strain.name", phenotype="MouseWeight", is.female="is.female",
   sep="", FixedEffects = NULL,
   RandomEffects = NULL, 
   Models=Example.Piximus.Models, sigmasq.start = 1,  numChains = 5,  lengthChains=2500,
   burnin = 1,
   DIC.Only=FALSE,  tauPriorFile = Example.Piximus.tau.Prior.Info,
   LogTransform=TRUE,
   SaveAFDFile = "SaveAFDBackUpPiximusData.RData");   
   
## Now make those HPD Plots!
AFD$AllDiallelObs[[1]]$PlotHPD();
AFD$AllDiallelObs[[1]]$PlotHPD(wanted = c("aj", "Gender:aj"), plot.narrow = .4, plot.wide = .9,
  xlab="Confidence intervals");
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{ methods }
\keyword{ ts }% __ONLY ONE__ keyword per line
